@@ -15,20 +15,18 @@ import (
 func TestZJSONWriter(t *testing.T) {
 	const record = `{x:1}`
 	const expected = `
-{"type":"QueryChannelSet","value":{"channel_id":1}}
+{"type":"QueryChannelSet","value":{"channel":"main"}}
 {"type":{"kind":"record","id":30,"fields":[{"name":"x","type":{"kind":"primitive","name":"int64"}}]},"value":["1"]}
-{"type":"QueryChannelEnd","value":{"channel_id":1}}
+{"type":"QueryChannelEnd","value":{"channel":"main"}}
 {"type":"QueryError","value":{"error":"test.err"}}
 `
 	var buf bytes.Buffer
 	w := queryio.NewZJSONWriter(&buf)
-	err := w.WriteControl(api.QueryChannelSet{ChannelID: 1})
+	err := w.WriteControl(api.QueryChannelSet{Channel: "main"})
 	require.NoError(t, err)
-	arena := zed.NewArena()
-	defer arena.Unref()
-	err = w.Write(zson.MustParseValue(zed.NewContext(), arena, record))
+	err = w.Write(zson.MustParseValue(zed.NewContext(), record))
 	require.NoError(t, err)
-	err = w.WriteControl(api.QueryChannelEnd{ChannelID: 1})
+	err = w.WriteControl(api.QueryChannelEnd{Channel: "main"})
 	require.NoError(t, err)
 	err = w.WriteControl(api.QueryError{Error: "test.err"})
 	require.NoError(t, err)
