@@ -4,7 +4,6 @@ package ast
 
 import (
 	astzed "github.com/brimdata/zed/compiler/ast/zed"
-	"github.com/brimdata/zed/order"
 	"github.com/brimdata/zed/pkg/field"
 )
 
@@ -466,11 +465,11 @@ type (
 		Rparen     int    `json:"rparen"`
 	}
 	Sort struct {
-		Kind       string      `json:"kind" unpack:""`
-		KeywordPos int         `json:"keyword_pos"`
-		Args       []Expr      `json:"args"`
-		Order      order.Which `json:"order"`
-		NullsFirst bool        `json:"nullsfirst"`
+		Kind       string     `json:"kind" unpack:""`
+		KeywordPos int        `json:"keyword_pos"`
+		Reverse    bool       `json:"reverse"`
+		NullsFirst bool       `json:"nullsfirst"`
+		Args       []SortExpr `json:"args"`
 	}
 	Cut struct {
 		Kind       string      `json:"kind" unpack:""`
@@ -622,25 +621,35 @@ type (
 		Expr       Expr   `json:"expr"`
 		Text       string `json:"text"`
 	}
+	Output struct {
+		Kind       string `json:"kind" unpack:""`
+		KeywordPos int    `json:"keyword_pos"`
+		Name       *ID    `json:"name"`
+	}
+	Debug struct {
+		Kind       string `json:"kind" unpack:""`
+		KeywordPos int    `json:"keyword_pos"`
+		Expr       Expr   `json:"expr"`
+	}
 )
 
 // Source structure
 
 type (
 	File struct {
-		Kind       string   `json:"kind" unpack:""`
-		KeywordPos int      `json:"keyword_pos"`
-		Path       Pattern  `json:"path"`
-		Format     string   `json:"format"`
-		SortKey    *SortKey `json:"sort_key"`
-		EndPos     int      `json:"end_pos"`
+		Kind       string     `json:"kind" unpack:""`
+		KeywordPos int        `json:"keyword_pos"`
+		Path       Pattern    `json:"path"`
+		Format     string     `json:"format"`
+		SortKeys   []SortExpr `json:"sort_keys"`
+		EndPos     int        `json:"end_pos"`
 	}
 	HTTP struct {
 		Kind       string      `json:"kind" unpack:""`
 		KeywordPos int         `json:"keyword_pos"`
 		URL        Pattern     `json:"url"`
 		Format     string      `json:"format"`
-		SortKey    *SortKey    `json:"sort_key"`
+		SortKeys   []SortExpr  `json:"sort_keys"`
 		Method     string      `json:"method"`
 		Headers    *RecordExpr `json:"headers"`
 		Body       string      `json:"body"`
@@ -680,10 +689,18 @@ func (x *Pool) End() int { return x.EndPos }
 func (x *File) End() int { return x.EndPos }
 func (x *HTTP) End() int { return x.EndPos }
 
-type SortKey struct {
+type SortExpr struct {
 	Kind  string `json:"kind" unpack:""`
-	Keys  []Expr `json:"keys"`
-	Order string `json:"order"`
+	Expr  Expr   `json:"expr"`
+	Order *ID    `json:"order"`
+}
+
+func (s SortExpr) Pos() int { return s.Expr.Pos() }
+func (s SortExpr) End() int {
+	if s.Order != nil {
+		s.Order.End()
+	}
+	return s.Expr.End()
 }
 
 type Trunk struct {
@@ -775,6 +792,8 @@ func (*Yield) OpAST()        {}
 func (*Sample) OpAST()       {}
 func (*Load) OpAST()         {}
 func (*Assert) OpAST()       {}
+func (*Output) OpAST()       {}
+func (*Debug) OpAST()        {}
 
 func (x *Scope) Pos() int {
 	if x.Decls != nil {
@@ -810,6 +829,8 @@ func (x *Yield) Pos() int        { return x.KeywordPos }
 func (x *Sample) Pos() int       { return x.KeywordPos }
 func (x *Load) Pos() int         { return x.KeywordPos }
 func (x *Assert) Pos() int       { return x.KeywordPos }
+func (x *Output) Pos() int       { return x.KeywordPos }
+func (x *Debug) Pos() int        { return x.KeywordPos }
 
 func (x *Scope) End() int    { return x.Body.End() }
 func (x *Parallel) End() int { return x.Rparen }
@@ -918,6 +939,13 @@ func (x *Sample) End() int {
 }
 func (x *Load) End() int   { return x.EndPos }
 func (x *Assert) End() int { return x.Expr.End() }
+func (x *Output) End() int { return x.Name.End() }
+func (x *Debug) End() int {
+	if x.Expr != nil {
+		return x.Expr.End()
+	}
+	return x.KeywordPos + 6
+}
 
 // An Agg is an AST node that represents a aggregate function.  The Name
 // field indicates the aggregation method while the Expr field indicates
