@@ -35,9 +35,8 @@ type (
 		Kind string `json:"kind" unpack:""`
 	}
 	Cut struct {
-		Kind  string       `json:"kind" unpack:""`
-		Args  []Assignment `json:"args"`
-		Quiet bool         `json:"quiet"`
+		Kind string       `json:"kind" unpack:""`
+		Args []Assignment `json:"args"`
 	}
 	Drop struct {
 		Kind string `json:"kind" unpack:""`
@@ -86,6 +85,11 @@ type (
 		Expr  Expr        `json:"expr"`
 		Order order.Which `json:"order"`
 	}
+	Mirror struct {
+		Kind   string `json:"kind" unpack:""`
+		Main   Seq    `json:"main"`
+		Mirror Seq    `json:"mirror"`
+	}
 	Over struct {
 		Kind  string `json:"kind" unpack:""`
 		Defs  []Def  `json:"defs"`
@@ -93,6 +97,10 @@ type (
 		Vars  []Def  `json:"vars"`
 		Body  Seq    `json:"body"`
 	}
+	Output struct {
+		Kind string `json:"kind" unpack:""`
+		Name string `json:"name"`
+	}
 	Pass struct {
 		Kind string `json:"kind" unpack:""`
 	}
@@ -118,10 +126,10 @@ type (
 		Kind string `json:"kind" unpack:""`
 	}
 	Sort struct {
-		Kind       string      `json:"kind" unpack:""`
-		Args       []Expr      `json:"args"`
-		Order      order.Which `json:"order"`
-		NullsFirst bool        `json:"nullsfirst"`
+		Kind       string     `json:"kind" unpack:""`
+		Args       []SortExpr `json:"args"`
+		NullsFirst bool       `json:"nullsfirst"`
+		Reverse    bool       `json:"reverse"`
 	}
 	Summarize struct {
 		Kind         string       `json:"kind" unpack:""`
@@ -193,25 +201,25 @@ type (
 
 	// DefaultScan scans an input stream provided by the runtime.
 	DefaultScan struct {
-		Kind    string        `json:"kind" unpack:""`
-		Filter  Expr          `json:"filter"`
-		SortKey order.SortKey `json:"sort_key"`
+		Kind     string         `json:"kind" unpack:""`
+		Filter   Expr           `json:"filter"`
+		SortKeys order.SortKeys `json:"sort_keys"`
 	}
 	FileScan struct {
-		Kind    string        `json:"kind" unpack:""`
-		Path    string        `json:"path"`
-		Format  string        `json:"format"`
-		SortKey order.SortKey `json:"sort_key"`
-		Filter  Expr          `json:"filter"`
+		Kind     string         `json:"kind" unpack:""`
+		Path     string         `json:"path"`
+		Format   string         `json:"format"`
+		SortKeys order.SortKeys `json:"sort_keys"`
+		Filter   Expr           `json:"filter"`
 	}
 	HTTPScan struct {
-		Kind    string              `json:"kind" unpack:""`
-		URL     string              `json:"url"`
-		Format  string              `json:"format"`
-		SortKey order.SortKey       `json:"sort_key"`
-		Method  string              `json:"method"`
-		Headers map[string][]string `json:"headers"`
-		Body    string              `json:"body"`
+		Kind     string              `json:"kind" unpack:""`
+		URL      string              `json:"url"`
+		Format   string              `json:"format"`
+		SortKeys order.SortKeys      `json:"sort_keys"`
+		Method   string              `json:"method"`
+		Headers  map[string][]string `json:"headers"`
+		Body     string              `json:"body"`
 	}
 	PoolScan struct {
 		Kind   string      `json:"kind" unpack:""`
@@ -310,9 +318,11 @@ func (*Over) OpNode()      {}
 func (*Vectorize) OpNode() {}
 func (*Yield) OpNode()     {}
 func (*Merge) OpNode()     {}
+func (*Mirror) OpNode()    {}
 func (*Combine) OpNode()   {}
 func (*Scope) OpNode()     {}
 func (*Load) OpNode()      {}
+func (*Output) OpNode()    {}
 
 // NewFilter returns a filter node for e.
 func NewFilter(e Expr) *Filter {
