@@ -41,6 +41,8 @@ var unpacker = unpack.New(
 	MapCall{},
 	MapExpr{},
 	Merge{},
+	Mirror{},
+	Output{},
 	Over{},
 	OverExpr{},
 	Pass{},
