@@ -22,6 +22,7 @@ var unpacker = unpack.New(
 	Conditional{},
 	ConstDecl{},
 	Cut{},
+	Debug{},
 	astzed.DefValue{},
 	Drop{},
 	Explode{},
@@ -45,6 +46,7 @@ var unpacker = unpack.New(
 	Join{},
 	Load{},
 	Merge{},
+	Output{},
 	Over{},
 	Trunk{},
 	astzed.Map{},
