@@ -48,8 +48,9 @@ func (*Pool) Source()     {}
 func (*Path) Source()     {}
 
 type Channel struct {
+	Name            string         `json:"name"`
 	AggregationKeys field.List     `json:"aggregation_keys"`
-	Sort            *order.SortKey `json:"sort"`
+	Sort            order.SortKeys `json:"sort"`
 }
 
 func Analyze(ctx context.Context, query string, src *data.Source, head *lakeparse.Commitish) (*Info, error) {
@@ -64,10 +65,15 @@ func Analyze(ctx context.Context, query string, src *data.Source, head *lakepars
 		}
 		return nil, err
 	}
+	return AnalyzeDAG(ctx, entry, src, head)
+}
+
+func AnalyzeDAG(ctx context.Context, entry dag.Seq, src *data.Source, head *lakeparse.Commitish) (*Info, error) {
 	srcInferred := !semantic.HasSource(entry)
-	if err = semantic.AddDefaultSource(ctx, &entry, src, head); err != nil {
+	if err := semantic.AddDefaultSource(ctx, &entry, src, head); err != nil {
 		return nil, err
 	}
+	var err error
 	var info Info
 	if info.Sources, err = describeSources(ctx, src.Lake(), entry[0], srcInferred); err != nil {
 		return nil, err
@@ -77,17 +83,24 @@ func Analyze(ctx context.Context, query string, src *data.Source, head *lakepars
 		return nil, err
 	}
 	aggKeys := describeAggs(entry, []field.List{nil})
-	for i := range sortKeys {
-		// Convert SortKey to a pointer so a nil sort is encoded as null for
-		// JSON/ZSON.
-		var s *order.SortKey
-		if !sortKeys[i].IsNil() {
-			s = &sortKeys[i]
+	outputs := collectOutputs(entry)
+	m := make(map[string]int)
+	for i, s := range sortKeys {
+		name := outputs[i].Name
+		if k, ok := m[name]; ok {
+			// If output already exists, this means the outputs will be
+			// combined so nil everything out.
+			// XXX This is currently what happens but is this right?
+			c := &info.Channels[k]
+			c.Sort, c.AggregationKeys = nil, nil
+			continue
 		}
 		info.Channels = append(info.Channels, Channel{
+			Name:            name,
 			Sort:            s,
 			AggregationKeys: aggKeys[i],
 		})
+		m[name] = i
 	}
 	return &info, nil
 }
@@ -149,6 +162,8 @@ func describeAggs(seq dag.Seq, parents []field.List) []field.List {
 
 func describeOpAggs(op dag.Op, parents []field.List) []field.List {
 	switch op := op.(type) {
+	case *dag.Scope:
+		return describeAggs(op.Body, parents)
 	case *dag.Fork:
 		var aggs []field.List
 		for _, p := range op.Paths {
@@ -161,6 +176,9 @@ func describeOpAggs(op dag.Op, parents []field.List) []field.List {
 			aggs = append(aggs, describeAggs(p, []field.List{nil})...)
 		}
 		return aggs
+	case *dag.Mirror:
+		aggs := describeAggs(op.Main, []field.List{nil})
+		return append(aggs, describeAggs(op.Mirror, []field.List{nil})...)
 	case *dag.Summarize:
 		// The field list for aggregation with no keys is an empty slice and
 		// not nil.
@@ -176,3 +194,16 @@ func describeOpAggs(op dag.Op, parents []field.List) []field.List {
 	}
 	return parents
 }
+
+func collectOutputs(seq dag.Seq) []*dag.Output {
+	var outputs []*dag.Output
+	optimizer.Walk(seq, func(seq dag.Seq) dag.Seq {
+		if len(seq) > 0 {
+			if o, ok := seq[len(seq)-1].(*dag.Output); ok {
+				outputs = append(outputs, o)
+			}
+		}
+		return seq
+	})
+	return outputs
+}
