@@ -52,7 +52,7 @@ func (b *Builder) compileExpr(e dag.Expr) (expr.Evaluator, error) {
 	}
 	switch e := e.(type) {
 	case *dag.Literal:
-		val, err := zson.ParseValue(b.zctx(), b.arena, e.Value)
+		val, err := zson.ParseValue(b.zctx(), e.Value)
 		if err != nil {
 			return nil, err
 		}
@@ -194,7 +194,7 @@ func (b *Builder) compileConstCompare(e *dag.BinaryExpr) (expr.Evaluator, error)
 }
 
 func (b *Builder) compileSearch(search *dag.Search) (expr.Evaluator, error) {
-	val, err := zson.ParseValue(b.zctx(), b.arena, search.Value)
+	val, err := zson.ParseValue(b.zctx(), search.Value)
 	if err != nil {
 		return nil, err
 	}
@@ -337,7 +337,7 @@ func (b *Builder) compileCall(call dag.Call) (expr.Evaluator, error) {
 	if err != nil {
 		return nil, fmt.Errorf("%s(): bad argument: %w", call.Name, err)
 	}
-	return expr.NewCall(b.zctx(), fn, exprs), nil
+	return expr.NewCall(fn, exprs), nil
 }
 
 func (b *Builder) compileUDFCall(name string, body dag.Expr) (expr.Function, error) {
