@@ -1,7 +1,6 @@
 package kernel
 
 import (
-	"github.com/brimdata/zed"
 	"github.com/brimdata/zed/compiler/ast/dag"
 	"github.com/brimdata/zed/runtime/sam/expr"
 	"github.com/brimdata/zed/zbuf"
@@ -18,19 +17,14 @@ func (f *Filter) AsEvaluator() (expr.Evaluator, error) {
 	if f == nil {
 		return nil, nil
 	}
-	arena := zed.NewArena()
-	e, err := f.builder.clone(arena).compileExpr(f.pushdown)
-	if err != nil {
-		return nil, err
-	}
-	return &arenaEvaluator{e, arena}, nil
+	return f.builder.compileExpr(f.pushdown)
 }
 
 func (f *Filter) AsBufferFilter() (*expr.BufferFilter, error) {
 	if f == nil {
 		return nil, nil
 	}
-	return CompileBufferFilter(f.builder.rctx.Zctx, f.pushdown)
+	return CompileBufferFilter(f.builder.zctx(), f.pushdown)
 }
 
 type DeleteFilter struct {
@@ -41,12 +35,11 @@ func (f *DeleteFilter) AsEvaluator() (expr.Evaluator, error) {
 	if f == nil {
 		return nil, nil
 	}
-	arena := zed.NewArena()
 	// For a DeleteFilter Evaluator the pushdown gets wrapped in a unary !
 	// expression so we get all values that don't match. We also add a missing
 	// call so if the expression results in an error("missing") the value is
 	// kept.
-	e, err := f.builder.clone(arena).compileExpr(&dag.BinaryExpr{
+	return f.builder.compileExpr(&dag.BinaryExpr{
 		Kind: "BinaryExpr",
 		Op:   "or",
 		LHS: &dag.UnaryExpr{
@@ -60,17 +53,8 @@ func (f *DeleteFilter) AsEvaluator() (expr.Evaluator, error) {
 			Args: []dag.Expr{f.pushdown},
 		},
 	})
-	if err != nil {
-		return nil, err
-	}
-	return &arenaEvaluator{e, arena}, nil
 }
 
 func (f *DeleteFilter) AsBufferFilter() (*expr.BufferFilter, error) {
 	return nil, nil
 }
-
-type arenaEvaluator struct {
-	expr.Evaluator
-	arena *zed.Arena
-}
