@@ -15,7 +15,6 @@ import (
 	"github.com/brimdata/zed/compiler/optimizer"
 	"github.com/brimdata/zed/compiler/optimizer/demand"
 	"github.com/brimdata/zed/lake"
-	"github.com/brimdata/zed/order"
 	"github.com/brimdata/zed/pkg/field"
 	"github.com/brimdata/zed/runtime"
 	"github.com/brimdata/zed/runtime/sam/expr"
@@ -30,6 +29,7 @@ import (
 	"github.com/brimdata/zed/runtime/sam/op/load"
 	"github.com/brimdata/zed/runtime/sam/op/merge"
 	"github.com/brimdata/zed/runtime/sam/op/meta"
+	"github.com/brimdata/zed/runtime/sam/op/mirror"
 	"github.com/brimdata/zed/runtime/sam/op/pass"
 	"github.com/brimdata/zed/runtime/sam/op/shape"
 	"github.com/brimdata/zed/runtime/sam/op/sort"
@@ -56,7 +56,7 @@ type Builder struct {
 	source       *data.Source
 	readers      []zio.Reader
 	progress     *zbuf.Progress
-	arena        *zed.Arena // For zed.Values created during compilation.
+	channels     map[string][]zbuf.Puller
 	deletes      *sync.Map
 	udfs         map[string]dag.Expr
 	compiledUDFs map[string]*expr.UDF
@@ -64,8 +64,6 @@ type Builder struct {
 }
 
 func NewBuilder(rctx *runtime.Context, source *data.Source) *Builder {
-	arena := zed.NewArena()
-	rctx.KeepAlive(arena)
 	return &Builder{
 		rctx:   rctx,
 		mctx:   zed.NewContext(),
@@ -76,26 +74,32 @@ func NewBuilder(rctx *runtime.Context, source *data.Source) *Builder {
 			RecordsRead:    0,
 			RecordsMatched: 0,
 		},
-		arena:        arena,
+		channels:     make(map[string][]zbuf.Puller),
 		udfs:         make(map[string]dag.Expr),
 		compiledUDFs: make(map[string]*expr.UDF),
 	}
 }
 
-func (b *Builder) clone(arena *zed.Arena) *Builder {
-	bb := *b
-	bb.arena = arena
-	return &bb
-}
-
 // Build builds a flowgraph for seq.  If seq contains a dag.DefaultSource, it
 // will read from readers.
-func (b *Builder) Build(seq dag.Seq, readers ...zio.Reader) ([]zbuf.Puller, error) {
+func (b *Builder) Build(seq dag.Seq, readers ...zio.Reader) (map[string]zbuf.Puller, error) {
 	if !isEntry(seq) {
 		return nil, errors.New("internal error: DAG entry point is not a data source")
 	}
 	b.readers = readers
-	return b.compileSeq(seq, nil)
+
+	if _, err := b.compileSeq(seq, nil); err != nil {
+		return nil, err
+	}
+	channels := make(map[string]zbuf.Puller)
+	for key, pullers := range b.channels {
+		if len(pullers) == 1 {
+			channels[key] = pullers[0]
+		} else {
+			channels[key] = combine.New(b.rctx, pullers)
+		}
+	}
+	return channels, nil
 }
 
 func (b *Builder) BuildWithPuller(seq dag.Seq, parent vector.Puller) ([]vector.Puller, error) {
@@ -151,10 +155,7 @@ func (b *Builder) compileLeaf(o dag.Op, parent zbuf.Puller) (zbuf.Puller, error)
 			return nil, err
 		}
 		lhs, rhs := splitAssignments(assignments)
-		cutter := expr.NewCutter(b.rctx.Zctx, lhs, rhs)
-		if v.Quiet {
-			cutter.Quiet()
-		}
+		cutter := expr.NewCutter(b.zctx(), lhs, rhs)
 		return op.NewApplier(b.rctx, parent, cutter, b.resetters), nil
 	case *dag.Drop:
 		if len(v.Args) == 0 {
@@ -168,15 +169,19 @@ func (b *Builder) compileLeaf(o dag.Op, parent zbuf.Puller) (zbuf.Puller, error)
 			}
 			fields = append(fields, field.Path)
 		}
-		dropper := expr.NewDropper(b.rctx.Zctx, fields)
+		dropper := expr.NewDropper(b.zctx(), fields)
 		return op.NewApplier(b.rctx, parent, dropper, expr.Resetters{}), nil
 	case *dag.Sort:
 		b.resetResetters()
-		fields, err := b.compileExprs(v.Args)
-		if err != nil {
-			return nil, err
+		var sortExprs []expr.SortEvaluator
+		for _, s := range v.Args {
+			k, err := b.compileExpr(s.Key)
+			if err != nil {
+				return nil, err
+			}
+			sortExprs = append(sortExprs, expr.NewSortEvaluator(k, s.Order))
 		}
-		sort, err := sort.New(b.rctx, parent, fields, v.Order, v.NullsFirst, b.resetters)
+		sort, err := sort.New(b.rctx, parent, sortExprs, v.NullsFirst, v.Reverse, b.resetters)
 		if err != nil {
 			return nil, fmt.Errorf("compiling sort: %w", err)
 		}
@@ -203,38 +208,29 @@ func (b *Builder) compileLeaf(o dag.Op, parent zbuf.Puller) (zbuf.Puller, error)
 		if err != nil {
 			return nil, fmt.Errorf("compiling filter: %w", err)
 		}
-		return op.NewApplier(b.rctx, parent, expr.NewFilterApplier(b.rctx.Zctx, f), b.resetters), nil
+		return op.NewApplier(b.rctx, parent, expr.NewFilterApplier(b.zctx(), f), b.resetters), nil
 	case *dag.Top:
 		b.resetResetters()
 		fields, err := b.compileExprs(v.Args)
 		if err != nil {
 			return nil, fmt.Errorf("compiling top: %w", err)
 		}
-		return top.New(b.rctx.Zctx, parent, v.Limit, fields, v.Flush, b.resetters), nil
+		return top.New(b.zctx(), parent, v.Limit, fields, v.Flush, b.resetters), nil
 	case *dag.Put:
 		b.resetResetters()
 		clauses, err := b.compileAssignments(v.Args)
 		if err != nil {
 			return nil, err
 		}
-		putter := expr.NewPutter(b.rctx.Zctx, clauses)
+		putter := expr.NewPutter(b.zctx(), clauses)
 		return op.NewApplier(b.rctx, parent, putter, b.resetters), nil
 	case *dag.Rename:
 		b.resetResetters()
-		var srcs, dsts []*expr.Lval
-		for _, a := range v.Args {
-			src, err := b.compileLval(a.RHS)
-			if err != nil {
-				return nil, err
-			}
-			dst, err := b.compileLval(a.LHS)
-			if err != nil {
-				return nil, err
-			}
-			srcs = append(srcs, src)
-			dsts = append(dsts, dst)
+		srcs, dsts, err := b.compileAssignmentsToLvals(v.Args)
+		if err != nil {
+			return nil, err
 		}
-		renamer := expr.NewRenamer(b.rctx.Zctx, srcs, dsts)
+		renamer := expr.NewRenamer(b.zctx(), srcs, dsts)
 		return op.NewApplier(b.rctx, parent, renamer, b.resetters), nil
 	case *dag.Fuse:
 		return fuse.New(b.rctx, parent)
@@ -245,7 +241,7 @@ func (b *Builder) compileLeaf(o dag.Op, parent zbuf.Puller) (zbuf.Puller, error)
 	case *dag.Merge:
 		return nil, errors.New("merge: multiple upstream paths required")
 	case *dag.Explode:
-		typ, err := zson.ParseType(b.rctx.Zctx, v.Type)
+		typ, err := zson.ParseType(b.zctx(), v.Type)
 		if err != nil {
 			return nil, err
 		}
@@ -254,7 +250,7 @@ func (b *Builder) compileLeaf(o dag.Op, parent zbuf.Puller) (zbuf.Puller, error)
 		if err != nil {
 			return nil, err
 		}
-		return explode.New(b.rctx.Zctx, parent, args, typ, v.As, b.resetters)
+		return explode.New(b.zctx(), parent, args, typ, v.As, b.resetters)
 	case *dag.Over:
 		return b.compileOver(parent, v)
 	case *dag.Yield:
@@ -271,24 +267,24 @@ func (b *Builder) compileLeaf(o dag.Op, parent zbuf.Puller) (zbuf.Puller, error)
 		}
 		return b.compilePoolScan(v)
 	case *dag.PoolMetaScan:
-		return meta.NewPoolMetaScanner(b.rctx.Context, b.rctx.Zctx, b.source.Lake(), v.ID, v.Meta)
+		return meta.NewPoolMetaScanner(b.rctx.Context, b.zctx(), b.source.Lake(), v.ID, v.Meta)
 	case *dag.CommitMetaScan:
 		var pruner expr.Evaluator
 		if v.Tap && v.KeyPruner != nil {
 			var err error
-			pruner, err = b.compileExpr(v.KeyPruner)
+			pruner, err = compileExpr(v.KeyPruner)
 			if err != nil {
 				return nil, err
 			}
 		}
-		return meta.NewCommitMetaScanner(b.rctx.Context, b.rctx.Zctx, b.source.Lake(), v.Pool, v.Commit, v.Meta, pruner)
+		return meta.NewCommitMetaScanner(b.rctx.Context, b.zctx(), b.source.Lake(), v.Pool, v.Commit, v.Meta, pruner)
 	case *dag.LakeMetaScan:
-		return meta.NewLakeMetaScanner(b.rctx.Context, b.rctx.Zctx, b.source.Lake(), v.Meta)
+		return meta.NewLakeMetaScanner(b.rctx.Context, b.zctx(), b.source.Lake(), v.Meta)
 	case *dag.HTTPScan:
 		body := strings.NewReader(v.Body)
-		return b.source.OpenHTTP(b.rctx.Context, b.rctx.Zctx, v.URL, v.Format, v.Method, v.Headers, body, demand.All())
+		return b.source.OpenHTTP(b.rctx.Context, b.zctx(), v.URL, v.Format, v.Method, v.Headers, body, demand.All())
 	case *dag.FileScan:
-		return b.source.Open(b.rctx.Context, b.rctx.Zctx, v.Path, v.Format, b.PushdownOf(v.Filter), demand.All())
+		return b.source.Open(b.rctx.Context, b.zctx(), v.Path, v.Format, b.PushdownOf(v.Filter), demand.All())
 	case *dag.DefaultScan:
 		pushdown := b.PushdownOf(v.Filter)
 		if len(b.readers) == 1 {
@@ -313,7 +309,7 @@ func (b *Builder) compileLeaf(o dag.Op, parent zbuf.Puller) (zbuf.Puller, error)
 		}
 		var pruner expr.Evaluator
 		if v.KeyPruner != nil {
-			pruner, err = b.compileExpr(v.KeyPruner)
+			pruner, err = compileExpr(v.KeyPruner)
 			if err != nil {
 				return nil, err
 			}
@@ -328,7 +324,7 @@ func (b *Builder) compileLeaf(o dag.Op, parent zbuf.Puller) (zbuf.Puller, error)
 		}
 		var pruner expr.Evaluator
 		if v.KeyPruner != nil {
-			pruner, err = b.compileExpr(v.KeyPruner)
+			pruner, err = compileExpr(v.KeyPruner)
 			if err != nil {
 				return nil, err
 			}
@@ -341,7 +337,7 @@ func (b *Builder) compileLeaf(o dag.Op, parent zbuf.Puller) (zbuf.Puller, error)
 		}
 		var pruner expr.Evaluator
 		if v.KeyPruner != nil {
-			pruner, err = b.compileExpr(v.KeyPruner)
+			pruner, err = compileExpr(v.KeyPruner)
 			if err != nil {
 				return nil, err
 			}
@@ -375,6 +371,9 @@ func (b *Builder) compileLeaf(o dag.Op, parent zbuf.Puller) (zbuf.Puller, error)
 		}
 		//XXX
 		return nil, errors.New("dag.Vectorize must begin with SeqScan")
+	case *dag.Output:
+		b.channels[v.Name] = append(b.channels[v.Name], parent)
+		return parent, nil
 	default:
 		return nil, fmt.Errorf("unknown DAG operator type: %v", v)
 	}
@@ -439,6 +438,23 @@ func (b *Builder) compileAssignments(assignments []dag.Assignment) ([]expr.Assig
 	return keys, nil
 }
 
+func (b *Builder) compileAssignmentsToLvals(assignments []dag.Assignment) ([]*expr.Lval, []*expr.Lval, error) {
+	var srcs, dsts []*expr.Lval
+	for _, a := range assignments {
+		src, err := b.compileLval(a.RHS)
+		if err != nil {
+			return nil, nil, err
+		}
+		dst, err := b.compileLval(a.LHS)
+		if err != nil {
+			return nil, nil, err
+		}
+		srcs = append(srcs, src)
+		dsts = append(dsts, dst)
+	}
+	return srcs, dsts, nil
+}
+
 func splitAssignments(assignments []expr.Assignment) ([]*expr.Lval, []expr.Evaluator) {
 	n := len(assignments)
 	lhs := make([]*expr.Lval, 0, n)
@@ -516,6 +532,23 @@ func (b *Builder) compileScatter(par *dag.Scatter, parents []zbuf.Puller) ([]zbu
 	return ops, nil
 }
 
+func (b *Builder) compileMirror(m *dag.Mirror, parents []zbuf.Puller) ([]zbuf.Puller, error) {
+	parent := parents[0]
+	if len(parents) > 1 {
+		parent = combine.New(b.rctx, parents)
+	}
+	o := mirror.New(b.rctx, parent)
+	main, err := b.compileSeq(m.Main, []zbuf.Puller{o})
+	if err != nil {
+		return nil, err
+	}
+	mirrored, err := b.compileSeq(m.Mirror, []zbuf.Puller{o.Mirrored()})
+	if err != nil {
+		return nil, err
+	}
+	return append(main, mirrored...), nil
+}
+
 func (b *Builder) compileExprSwitch(swtch *dag.Switch, parents []zbuf.Puller) ([]zbuf.Puller, error) {
 	parent := parents[0]
 	if len(parents) > 1 {
@@ -583,6 +616,8 @@ func (b *Builder) compile(o dag.Op, parents []zbuf.Puller) ([]zbuf.Puller, error
 		return b.compileFork(o, parents)
 	case *dag.Scatter:
 		return b.compileScatter(o, parents)
+	case *dag.Mirror:
+		return b.compileMirror(o, parents)
 	case *dag.Scope:
 		return b.compileScope(o, parents)
 	case *dag.Switch:
@@ -635,7 +670,7 @@ func (b *Builder) compile(o dag.Op, parents []zbuf.Puller) ([]zbuf.Puller, error
 		if err != nil {
 			return nil, err
 		}
-		cmp := expr.NewComparator(true, o.Order == order.Desc, e).WithMissingAsNull()
+		cmp := expr.NewComparator(true, expr.NewSortEvaluator(e, o.Order)).WithMissingAsNull()
 		return []zbuf.Puller{merge.New(b.rctx, parents, cmp.Compare, b.resetters)}, nil
 	case *dag.Combine:
 		return []zbuf.Puller{combine.New(b.rctx, parents)}, nil
@@ -693,22 +728,25 @@ func (b *Builder) evalAtCompileTime(in dag.Expr) (val zed.Value, err error) {
 	if err != nil {
 		return zed.Null, err
 	}
-	missing := b.zctx().Missing(b.arena)
 	// Catch panic as the runtime will panic if there is a
 	// reference to a var not in scope, a field access null this, etc.
 	defer func() {
 		if recover() != nil {
-			val = missing
+			val = b.zctx().Missing()
 		}
 	}()
-	return e.Eval(expr.NewContext(b.arena), missing), nil
+	return e.Eval(expr.NewContext(), b.zctx().Missing()), nil
 }
 
-func EvalAtCompileTime(zctx *zed.Context, arena *zed.Arena, in dag.Expr) (zed.Value, error) {
+func compileExpr(in dag.Expr) (expr.Evaluator, error) {
+	b := NewBuilder(runtime.NewContext(context.Background(), zed.NewContext()), nil)
+	return b.compileExpr(in)
+}
+
+func EvalAtCompileTime(zctx *zed.Context, in dag.Expr) (val zed.Value, err error) {
 	// We pass in a nil adaptor, which causes a panic for anything adaptor
 	// related, which is not currently allowed in an expression sub-query.
 	b := NewBuilder(runtime.NewContext(context.Background(), zctx), nil)
-	b.arena = arena
 	return b.evalAtCompileTime(in)
 }
 
