@@ -5,10 +5,8 @@ import (
 	"encoding/json"
 	"fmt"
 	"os"
-	"os/exec"
 	"path/filepath"
 	"regexp"
-	"runtime"
 	"strings"
 	"testing"
 
@@ -41,14 +39,8 @@ func searchForZed() ([]string, error) {
 	return zed, err
 }
 
-func parsePEGjs(z string) ([]byte, error) {
-	cmd := exec.Command("node", "run.js", "-e", "start")
-	cmd.Stdin = strings.NewReader(z)
-	return cmd.Output()
-}
-
 func parseOp(z string) ([]byte, error) {
-	o, err := compiler.Parse(z)
+	o, _, err := compiler.Parse(z)
 	if err != nil {
 		return nil, err
 	}
@@ -63,11 +55,9 @@ func parsePigeon(z string) ([]byte, error) {
 	return json.Marshal(ast)
 }
 
-// testZed parses the Zed query in line by both the Go and JavaScript
-// parsers.  It checks both that the parse is successful and that the
-// two resulting ASTs are equivalent.  On the go side, we take a round
-// trip through json marshal and unmarshal to turn the parse-tree types
-// into generic JSON types.
+// testZed checks both that the parse is successful and that the
+// two resulting ASTs from the round trip through json marshal and
+// unmarshal are equivalent.
 func testZed(t *testing.T, line string) {
 	pigeonJSON, err := parsePigeon(line)
 	assert.NoError(t, err, "parsePigeon: %q", line)
@@ -76,12 +66,6 @@ func testZed(t *testing.T, line string) {
 	assert.NoError(t, err, "parseOp: %q", line)
 
 	assert.JSONEq(t, string(pigeonJSON), string(astJSON), "pigeon and AST mismatch: %q", line)
-
-	if runtime.GOOS != "windows" {
-		pegJSON, err := parsePEGjs(line)
-		assert.NoError(t, err, "parsePEGjs: %q", line)
-		assert.JSONEq(t, string(pigeonJSON), string(pegJSON), "pigeon and PEGjs mismatch: %q", line)
-	}
 }
 
 func TestValid(t *testing.T) {
