@@ -24,7 +24,10 @@ import (
 
 func (a *analyzer) semSeq(seq ast.Seq) dag.Seq {
 	var converted dag.Seq
-	for _, op := range seq {
+	for k, op := range seq {
+		if d, ok := op.(*ast.Debug); ok {
+			return a.semDebugOp(d, seq[k+1:], converted)
+		}
 		converted = a.semOp(op, converted)
 	}
 	return converted
@@ -86,16 +89,13 @@ func (a *analyzer) semOpSource(source ast.Source, out dag.Seq) dag.Seq {
 func (a *analyzer) semSource(source ast.Source) []dag.Op {
 	switch s := source.(type) {
 	case *ast.File:
-		sortKey, err := semSortKey(s.SortKey)
-		if err != nil {
-			a.error(source, err)
-		}
 		var path string
 		switch p := s.Path.(type) {
 		case *ast.QuotedString:
 			path = p.Text
 		case *ast.String:
 			// This can be either a reference to a constant or a string.
+			var err error
 			if path, err = a.maybeStringConst(p.Text); err != nil {
 				a.error(s.Path, err)
 			}
@@ -104,25 +104,21 @@ func (a *analyzer) semSource(source ast.Source) []dag.Op {
 		}
 		return []dag.Op{
 			&dag.FileScan{
-				Kind:    "FileScan",
-				Path:    path,
-				Format:  s.Format,
-				SortKey: sortKey,
+				Kind:     "FileScan",
+				Path:     path,
+				Format:   s.Format,
+				SortKeys: a.semSortKeys(s.SortKeys),
 			},
 		}
 	case *ast.HTTP:
-		sortKey, err := semSortKey(s.SortKey)
-		if err != nil {
-			a.error(source, err)
-		}
 		var headers map[string][]string
 		if s.Headers != nil {
 			expr := a.semExpr(s.Headers)
-			val, err := kernel.EvalAtCompileTime(a.zctx, a.arena, expr)
+			val, err := kernel.EvalAtCompileTime(a.zctx, expr)
 			if err != nil {
 				a.error(s.Headers, err)
 			} else {
-				headers, err = unmarshalHeaders(a.arena, val)
+				headers, err = unmarshalHeaders(val)
 				if err != nil {
 					a.error(s.Headers, err)
 				}
@@ -134,6 +130,7 @@ func (a *analyzer) semSource(source ast.Source) []dag.Op {
 			url = p.Text
 		case *ast.String:
 			// This can be either a reference to a constant or a string.
+			var err error
 			if url, err = a.maybeStringConst(p.Text); err != nil {
 				a.error(s.URL, err)
 				// Set url so we don't report an error for this twice.
@@ -147,13 +144,13 @@ func (a *analyzer) semSource(source ast.Source) []dag.Op {
 		}
 		return []dag.Op{
 			&dag.HTTPScan{
-				Kind:    "HTTPScan",
-				URL:     url,
-				Format:  s.Format,
-				SortKey: sortKey,
-				Method:  s.Method,
-				Headers: headers,
-				Body:    s.Body,
+				Kind:     "HTTPScan",
+				URL:      url,
+				Format:   s.Format,
+				SortKeys: a.semSortKeys(s.SortKeys),
+				Method:   s.Method,
+				Headers:  headers,
+				Body:     s.Body,
 			},
 		}
 	case *ast.Pool:
@@ -170,7 +167,7 @@ func (a *analyzer) semSource(source ast.Source) []dag.Op {
 	}
 }
 
-func unmarshalHeaders(arena *zed.Arena, val zed.Value) (map[string][]string, error) {
+func unmarshalHeaders(val zed.Value) (map[string][]string, error) {
 	if !zed.IsRecordType(val.Type()) {
 		return nil, errors.New("headers value must be a record")
 	}
@@ -179,7 +176,7 @@ func unmarshalHeaders(arena *zed.Arena, val zed.Value) (map[string][]string, err
 		if inner := zed.InnerType(f.Type); inner == nil || inner.ID() != zed.IDString {
 			return nil, errors.New("headers field value must be an array or set of strings")
 		}
-		fieldVal := val.DerefByColumn(arena, i)
+		fieldVal := val.DerefByColumn(i)
 		if fieldVal == nil {
 			continue
 		}
@@ -192,25 +189,31 @@ func unmarshalHeaders(arena *zed.Arena, val zed.Value) (map[string][]string, err
 	return headers, nil
 }
 
-// XXX At some point SortKey should be an ast.Node but for now just have this
-// return and error and let the parent log the semantic error.
-func semSortKey(p *ast.SortKey) (order.SortKey, error) {
-	if p == nil || p.Keys == nil {
-		return order.Nil, nil
-	}
-	var keys field.List
-	for _, key := range p.Keys {
-		this := DotExprToFieldPath(key)
-		if this == nil {
-			return order.Nil, fmt.Errorf("bad key expr of type %T in file operator", key)
+func (a *analyzer) semSortKeys(sortExprs []ast.SortExpr) order.SortKeys {
+	var sortKeys order.SortKeys
+	for _, e := range sortExprs {
+		s := a.semSortExpr(e)
+		switch key := s.Key.(type) {
+		case *dag.This:
+			sortKeys = append(sortKeys, order.NewSortKey(s.Order, key.Path))
+		case *dag.BadExpr: // ignore so we don't report double errors
+		default:
+			a.error(e.Expr, errors.New("field required in sort expression"))
 		}
-		keys = append(keys, this.Path)
 	}
-	which, err := order.Parse(p.Order)
-	if err != nil {
-		return order.Nil, err
+	return sortKeys
+}
+
+func (a *analyzer) semSortExpr(s ast.SortExpr) dag.SortExpr {
+	e := a.semExpr(s.Expr)
+	o := order.Asc
+	if s.Order != nil {
+		var err error
+		if o, err = order.Parse(s.Order.Name); err != nil {
+			a.error(s.Order, err)
+		}
 	}
-	return order.NewSortKey(which, keys), nil
+	return dag.SortExpr{Key: e, Order: o}
 }
 
 func (a *analyzer) semPool(p *ast.Pool) []dag.Op {
@@ -257,7 +260,7 @@ func (a *analyzer) maybeStringConst(name string) (string, error) {
 	if !ok {
 		return "", fmt.Errorf("%s: string value required", name)
 	}
-	val := zson.MustParseValue(a.zctx, a.arena, l.Value)
+	val := zson.MustParseValue(a.zctx, l.Value)
 	if val.Type().ID() != zed.IDString {
 		return "", fmt.Errorf("%s: string value required", name)
 	}
@@ -387,6 +390,25 @@ func (a *analyzer) semScope(op *ast.Scope) *dag.Scope {
 	}
 }
 
+func (a *analyzer) semDebugOp(o *ast.Debug, mainAst ast.Seq, in dag.Seq) dag.Seq {
+	output := &dag.Output{Kind: "Output", Name: "debug"}
+	a.outputs[output] = o
+	e := a.semExprNullable(o.Expr)
+	if e == nil {
+		e = &dag.This{Kind: "This"}
+	}
+	y := &dag.Yield{Kind: "Yield", Exprs: []dag.Expr{e}}
+	main := a.semSeq(mainAst)
+	if len(main) == 0 {
+		main.Append(&dag.Pass{Kind: "Pass"})
+	}
+	return append(in, &dag.Mirror{
+		Kind:   "Mirror",
+		Main:   main,
+		Mirror: dag.Seq{y, output},
+	})
+}
+
 // semOp does a semantic analysis on a flowgraph to an
 // intermediate representation that can be compiled into the runtime
 // object.  Currently, it only replaces the group-by duration with
@@ -489,19 +511,22 @@ func (a *analyzer) semOp(o ast.Op, seq dag.Seq) dag.Seq {
 			Args: args,
 		})
 	case *ast.Sort:
-		exprs := a.semExprs(o.Args)
+		var sortExprs []dag.SortExpr
+		for _, arg := range o.Args {
+			sortExprs = append(sortExprs, a.semSortExpr(arg))
+		}
 		return append(seq, &dag.Sort{
 			Kind:       "Sort",
-			Args:       exprs,
-			Order:      o.Order,
+			Args:       sortExprs,
 			NullsFirst: o.NullsFirst,
+			Reverse:    o.Reverse,
 		})
 	case *ast.Head:
 		val := zed.NewInt64(1)
 		if o.Count != nil {
 			expr := a.semExpr(o.Count)
 			var err error
-			if val, err = kernel.EvalAtCompileTime(a.zctx, a.arena, expr); err != nil {
+			if val, err = kernel.EvalAtCompileTime(a.zctx, expr); err != nil {
 				a.error(o.Count, err)
 				return append(seq, badOp())
 			}
@@ -522,7 +547,7 @@ func (a *analyzer) semOp(o ast.Op, seq dag.Seq) dag.Seq {
 		if o.Count != nil {
 			expr := a.semExpr(o.Count)
 			var err error
-			if val, err = kernel.EvalAtCompileTime(a.zctx, a.arena, expr); err != nil {
+			if val, err = kernel.EvalAtCompileTime(a.zctx, expr); err != nil {
 				a.error(o.Count, err)
 				return append(seq, badOp())
 			}
@@ -561,7 +586,7 @@ func (a *analyzer) semOp(o ast.Op, seq dag.Seq) dag.Seq {
 		limit := 1
 		if o.Limit != nil {
 			l := a.semExpr(o.Limit)
-			val, err := kernel.EvalAtCompileTime(a.zctx, a.arena, l)
+			val, err := kernel.EvalAtCompileTime(a.zctx, l)
 			if err != nil {
 				a.error(o.Limit, err)
 				return append(seq, badOp())
@@ -791,6 +816,10 @@ func (a *analyzer) semOp(o ast.Op, seq dag.Seq) dag.Seq {
 			Message: o.Message,
 			Meta:    o.Meta,
 		})
+	case *ast.Output:
+		out := &dag.Output{Kind: "Output", Name: o.Name.Name}
+		a.outputs[out] = o
+		return append(seq, out)
 	}
 	panic(fmt.Errorf("semantic transform: unknown AST operator type: %T", o))
 }
@@ -839,7 +868,7 @@ func (a *analyzer) semDecls(decls []ast.Decl) ([]dag.Def, []*dag.Func) {
 
 func (a *analyzer) semConstDecl(c *ast.ConstDecl) dag.Def {
 	e := a.semExpr(c.Expr)
-	if err := a.scope.DefineConst(a.zctx, a.arena, c.Name, e); err != nil {
+	if err := a.scope.DefineConst(a.zctx, c.Name, e); err != nil {
 		a.error(c, err)
 	}
 	return dag.Def{
@@ -858,7 +887,7 @@ func (a *analyzer) semTypeDecl(d *ast.TypeDecl) dag.Def {
 		Kind:  "Literal",
 		Value: fmt.Sprintf("<%s=%s>", zson.QuotedName(d.Name.Name), typ),
 	}
-	if err := a.scope.DefineConst(a.zctx, a.arena, d.Name, e); err != nil {
+	if err := a.scope.DefineConst(a.zctx, d.Name, e); err != nil {
 		a.error(d.Name, err)
 	}
 	return dag.Def{Name: d.Name.Name, Expr: e}
@@ -1084,7 +1113,7 @@ func (a *analyzer) maybeConvertUserOp(call *ast.Call) dag.Seq {
 		e := a.semExpr(arg)
 		// Transform non-path arguments into literals.
 		if _, ok := e.(*dag.This); !ok {
-			val, err := kernel.EvalAtCompileTime(a.zctx, a.arena, e)
+			val, err := kernel.EvalAtCompileTime(a.zctx, e)
 			if err != nil {
 				a.error(arg, err)
 				exprs[i] = badExpr()
