@@ -162,10 +162,6 @@ func (r *Root) readLakeMagic(ctx context.Context) error {
 	if err != nil {
 		return err
 	}
-	var magic LakeMagic
-	if err := zson.UnmarshalZNG(nil, nil, *val, &magic); err != nil {
-		return fmt.Errorf("corrupt lake version file: %w", err)
-	}
 	last, err := zr.Read()
 	if err != nil {
 		return err
@@ -173,6 +169,10 @@ func (r *Root) readLakeMagic(ctx context.Context) error {
 	if last != nil {
 		return fmt.Errorf("corrupt lake version file: more than one Zed value at %s", zson.String(last))
 	}
+	var magic LakeMagic
+	if err := zson.UnmarshalZNG(*val, &magic); err != nil {
+		return fmt.Errorf("corrupt lake version file: %w", err)
+	}
 	if magic.Magic != LakeMagicString {
 		return fmt.Errorf("corrupt lake version file: magic %q should be %q", magic.Magic, LakeMagicString)
 	}
@@ -182,17 +182,17 @@ func (r *Root) readLakeMagic(ctx context.Context) error {
 	return nil
 }
 
-func (r *Root) BatchifyPools(ctx context.Context, zctx *zed.Context, arena *zed.Arena, f expr.Evaluator) ([]zed.Value, error) {
+func (r *Root) BatchifyPools(ctx context.Context, zctx *zed.Context, f expr.Evaluator) ([]zed.Value, error) {
 	m := zson.NewZNGMarshalerWithContext(zctx)
 	m.Decorate(zson.StylePackage)
 	pools, err := r.ListPools(ctx)
 	if err != nil {
 		return nil, err
 	}
-	ectx := expr.NewContext(arena)
+	ectx := expr.NewContext()
 	var vals []zed.Value
 	for k := range pools {
-		rec, err := m.Marshal(arena, &pools[k])
+		rec, err := m.Marshal(&pools[k])
 		if err != nil {
 			return nil, err
 		}
@@ -203,7 +203,7 @@ func (r *Root) BatchifyPools(ctx context.Context, zctx *zed.Context, arena *zed.
 	return vals, nil
 }
 
-func (r *Root) BatchifyBranches(ctx context.Context, zctx *zed.Context, arena *zed.Arena, f expr.Evaluator) ([]zed.Value, error) {
+func (r *Root) BatchifyBranches(ctx context.Context, zctx *zed.Context, f expr.Evaluator) ([]zed.Value, error) {
 	m := zson.NewZNGMarshalerWithContext(zctx)
 	m.Decorate(zson.StylePackage)
 	poolRefs, err := r.ListPools(ctx)
@@ -221,7 +221,7 @@ func (r *Root) BatchifyBranches(ctx context.Context, zctx *zed.Context, arena *z
 			}
 			return nil, err
 		}
-		vals, err = pool.BatchifyBranches(ctx, zctx, arena, vals, m, f)
+		vals, err = pool.BatchifyBranches(ctx, zctx, vals, m, f)
 		if err != nil {
 			return nil, err
 		}
@@ -261,28 +261,28 @@ func (r *Root) CommitObject(ctx context.Context, poolID ksuid.KSUID, branchName
 	return branchRef.Commit, nil
 }
 
-func (r *Root) SortKey(ctx context.Context, src dag.Op) order.SortKey {
+func (r *Root) SortKeys(ctx context.Context, src dag.Op) order.SortKeys {
 	switch src := src.(type) {
 	case *dag.Lister:
 		if config, err := r.pools.LookupByID(ctx, src.Pool); err == nil {
-			return config.SortKey
+			return config.SortKeys
 		}
 	case *dag.SeqScan:
 		if config, err := r.pools.LookupByID(ctx, src.Pool); err == nil {
-			return config.SortKey
+			return config.SortKeys
 		}
 	case *dag.PoolScan:
 		if config, err := r.pools.LookupByID(ctx, src.ID); err == nil {
-			return config.SortKey
+			return config.SortKeys
 		}
 	case *dag.CommitMetaScan:
 		if src.Tap {
 			if config, err := r.pools.LookupByID(ctx, src.Pool); err == nil {
-				return config.SortKey
+				return config.SortKeys
 			}
 		}
 	}
-	return order.Nil
+	return nil
 }
 
 func (r *Root) OpenPool(ctx context.Context, id ksuid.KSUID) (*Pool, error) {
@@ -314,7 +314,7 @@ func (r *Root) RenamePool(ctx context.Context, id ksuid.KSUID, newName string) e
 	return r.pools.Rename(ctx, id, newName)
 }
 
-func (r *Root) CreatePool(ctx context.Context, name string, sortKey order.SortKey, seekStride int, thresh int64) (*Pool, error) {
+func (r *Root) CreatePool(ctx context.Context, name string, sortKeys order.SortKeys, seekStride int, thresh int64) (*Pool, error) {
 	if name == "HEAD" {
 		return nil, fmt.Errorf("pool cannot be named %q", name)
 	}
@@ -324,10 +324,10 @@ func (r *Root) CreatePool(ctx context.Context, name string, sortKey order.SortKe
 	if thresh == 0 {
 		thresh = data.DefaultThreshold
 	}
-	if len(sortKey.Keys) > 1 {
+	if len(sortKeys) > 1 {
 		return nil, errors.New("multiple pool keys not supported")
 	}
-	config := pools.NewConfig(name, sortKey, thresh, seekStride)
+	config := pools.NewConfig(name, sortKeys, thresh, seekStride)
 	if err := CreatePool(ctx, r.engine, r.logger, r.path, config); err != nil {
 		return nil, err
 	}
