@@ -1,8 +1,6 @@
 package exec
 
 import (
-	"io"
-
 	"github.com/brimdata/zed/runtime"
 	"github.com/brimdata/zed/zbuf"
 	"github.com/brimdata/zed/zio"
@@ -31,14 +29,6 @@ func (q *Query) AsReader() zio.Reader {
 	return zbuf.PullerReader(q)
 }
 
-func (q *Query) AsProgressReadCloser() zbuf.ProgressReadCloser {
-	return struct {
-		zio.Reader
-		io.Closer
-		zbuf.Meter
-	}{q.AsReader(), q, q}
-}
-
 func (q *Query) Progress() zbuf.Progress {
 	return q.meter.Progress()
 }
