@@ -28,12 +28,12 @@ func newMathReducer(f *anymath.Function) *mathReducer {
 	return &mathReducer{function: f}
 }
 
-func (m *mathReducer) Result(_ *zed.Context, arena *zed.Arena) zed.Value {
+func (m *mathReducer) Result(zctx *zed.Context) zed.Value {
 	if !m.hasval {
 		if m.math == nil {
 			return zed.Null
 		}
-		return arena.New(m.math.typ(), nil)
+		return zed.NewValue(m.math.typ(), nil)
 	}
 	return m.math.result()
 }
@@ -46,9 +46,7 @@ func (m *mathReducer) consumeVal(val zed.Value) {
 	var id int
 	if m.math != nil {
 		var err error
-		arena := zed.NewArena()
-		defer arena.Unref()
-		id, err = coerce.Promote(arena.New(m.math.typ(), nil), val)
+		id, err = coerce.Promote(zed.NewValue(m.math.typ(), nil), val)
 		if err != nil {
 			// Skip invalid values.
 			return
@@ -84,8 +82,8 @@ func (m *mathReducer) consumeVal(val zed.Value) {
 	m.math.consume(val)
 }
 
-func (m *mathReducer) ResultAsPartial(zctx *zed.Context, arena *zed.Arena) zed.Value {
-	return m.Result(zctx, arena)
+func (m *mathReducer) ResultAsPartial(*zed.Context) zed.Value {
+	return m.Result(nil)
 }
 
 func (m *mathReducer) ConsumeAsPartial(val zed.Value) {
