@@ -20,7 +20,6 @@ type Cutter struct {
 	droppers     map[string]*Dropper
 	dropperCache []*Dropper
 	dirty        bool
-	quiet        bool
 }
 
 // NewCutter returns a Cutter for fieldnames. If complement is true,
@@ -42,10 +41,6 @@ func NewCutter(zctx *zed.Context, fieldRefs []*Lval, fieldExprs []Evaluator) *Cu
 	}
 }
 
-func (c *Cutter) Quiet() {
-	c.quiet = true
-}
-
 func (c *Cutter) FoundCut() bool {
 	return c.dirty
 }
@@ -56,7 +51,7 @@ func (c *Cutter) FoundCut() bool {
 func (c *Cutter) Eval(ectx Context, in zed.Value) zed.Value {
 	rb, paths, err := c.lookupBuilder(ectx, in)
 	if err != nil {
-		return c.zctx.WrapError(ectx.Arena(), fmt.Sprintf("cut: %s", err), in)
+		return c.zctx.WrapError(fmt.Sprintf("cut: %s", err), in)
 	}
 	types := c.typeCache
 	rb.Reset()
@@ -81,7 +76,7 @@ func (c *Cutter) Eval(ectx Context, in zed.Value) zed.Value {
 	if err != nil {
 		panic(err)
 	}
-	rec := ectx.Arena().New(rb.Type(c.outTypes.Lookup(types), types), bytes)
+	rec := zed.NewValue(rb.Type(c.outTypes.Lookup(types), types), bytes)
 	for _, d := range droppers {
 		rec = d.Eval(ectx, rec)
 	}
