@@ -2,7 +2,6 @@ package function
 
 import (
 	"github.com/brimdata/zed"
-	"github.com/brimdata/zed/runtime/sam/expr"
 	"github.com/brimdata/zed/zcode"
 )
 
@@ -19,8 +18,7 @@ func NewFields(zctx *zed.Context) *Fields {
 	}
 }
 
-func buildPath(typ *zed.TypeRecord, b *zcode.Builder, prefix []string) []string {
-	var out []string
+func buildPath(typ *zed.TypeRecord, b *zcode.Builder, prefix []string) {
 	for _, f := range typ.Fields {
 		if typ, ok := zed.TypeUnder(f.Type).(*zed.TypeRecord); ok {
 			buildPath(typ, b, append(prefix, f.Name))
@@ -33,20 +31,17 @@ func buildPath(typ *zed.TypeRecord, b *zcode.Builder, prefix []string) []string
 			b.EndContainer()
 		}
 	}
-	return out
 }
 
-func (f *Fields) Call(ectx expr.Context, args []zed.Value) zed.Value {
-	arena := ectx.Arena()
-	subjectVal := args[0]
+func (f *Fields) Call(_ zed.Allocator, args []zed.Value) zed.Value {
+	subjectVal := args[0].Under()
 	typ := f.recordType(subjectVal)
 	if typ == nil {
-		return f.zctx.Missing(ectx.Arena())
+		return f.zctx.Missing()
 	}
-	//XXX should have a way to append into allocator
 	var b zcode.Builder
 	buildPath(typ, &b, nil)
-	return arena.New(f.typ, b.Bytes())
+	return zed.NewValue(f.typ, b.Bytes())
 }
 
 func (f *Fields) recordType(val zed.Value) *zed.TypeRecord {
