@@ -59,23 +59,18 @@ type Aggregator struct {
 	valueCompare   expr.CompareFn   // to compare primary group keys for early key output
 	keyCompare     expr.CompareFn   // compare the first key (used when input sorted)
 	keysComparator *expr.Comparator // compare all keys
-	maxTableKey    zed.Value
-	maxSpillKey    zed.Value
+	maxTableKey    *zed.Value
+	maxSpillKey    *zed.Value
 	inputDir       order.Direction
 	spiller        *spill.MergeSort
 	partialsIn     bool
 	partialsOut    bool
-
-	ectx             *expr.Ctx
-	maxTableKeyArena *zed.Arena
-	maxSpillKeyArena *zed.Arena
 }
 
 type Row struct {
-	keyType       int
-	groupval      zed.Value // for sorting when input sorted
-	groupvalArena *zed.Arena
-	reducers      valRow
+	keyType  int
+	groupval zed.Value // for sorting when input sorted
+	reducers valRow
 }
 
 func NewAggregator(ctx context.Context, zctx *zed.Context, keyRefs, keyExprs, aggRefs []expr.Evaluator, aggs []*expr.Aggregator, builder *zed.RecordBuilder, limit int, inputDir order.Direction, partialsIn, partialsOut bool) (*Aggregator, error) {
@@ -84,8 +79,9 @@ func NewAggregator(ctx context.Context, zctx *zed.Context, keyRefs, keyExprs, ag
 	}
 	var keyCompare, valueCompare expr.CompareFn
 	nkeys := len(keyExprs)
+	o := order.Which(inputDir < 0)
 	if nkeys > 0 && inputDir != 0 {
-		valueCompare = expr.NewValueCompareFn(order.Which(inputDir < 0), true)
+		valueCompare = expr.NewValueCompareFn(o, true)
 		rs := expr.NewCompareFn(true, keyRefs[0])
 		if inputDir < 0 {
 			keyCompare = func(a, b zed.Value) int { return rs(b, a) }
@@ -93,6 +89,10 @@ func NewAggregator(ctx context.Context, zctx *zed.Context, keyRefs, keyExprs, ag
 			keyCompare = rs
 		}
 	}
+	var sortExprs []expr.SortEvaluator
+	for _, e := range keyRefs {
+		sortExprs = append(sortExprs, expr.NewSortEvaluator(e, o))
+	}
 	return &Aggregator{
 		ctx:            ctx,
 		zctx:           zctx,
@@ -110,14 +110,10 @@ func NewAggregator(ctx context.Context, zctx *zed.Context, keyRefs, keyExprs, ag
 		table:          make(map[string]*Row),
 		recordTypes:    make(map[int]*zed.TypeRecord),
 		keyCompare:     keyCompare,
-		keysComparator: expr.NewComparator(true, inputDir < 0, keyRefs...).WithMissingAsNull(),
+		keysComparator: expr.NewComparator(true, sortExprs...).WithMissingAsNull(),
 		valueCompare:   valueCompare,
 		partialsIn:     partialsIn,
 		partialsOut:    partialsOut,
-
-		ectx:             expr.NewContext(zed.NewArena()),
-		maxTableKeyArena: zed.NewArena(),
-		maxSpillKeyArena: zed.NewArena(),
 	}, nil
 }
 
@@ -331,14 +327,11 @@ func (a *Aggregator) Consume(batch zbuf.Batch, this zed.Value) error {
 	// structure at output time, which is the new approach that will be
 	// taken by the fix to #1701.
 
-	*a.ectx = *expr.NewContextWithVars(a.ectx.Arena(), batch.Vars())
-
 	types := a.typeCache[:0]
 	keyBytes := a.keyCache[:0]
 	var prim zed.Value
 	for i, keyExpr := range a.keyExprs {
-		a.ectx.Arena().Reset()
-		key := keyExpr.Eval(a.ectx, this)
+		key := keyExpr.Eval(batch, this)
 		if key.IsQuiet() {
 			return nil
 		}
@@ -365,20 +358,16 @@ func (a *Aggregator) Consume(batch zbuf.Batch, this zed.Value) error {
 		}
 		row = &Row{
 			keyType:  keyType,
+			groupval: prim,
 			reducers: newValRow(a.aggs),
 		}
-		if !prim.IsZero() {
-			row.groupvalArena = zed.NewArena()
-			row.groupval = prim.Copy(row.groupvalArena)
-		}
 		a.table[string(keyBytes)] = row
 	}
 
-	a.ectx.Arena().Reset()
 	if a.partialsIn {
-		row.reducers.consumeAsPartial(this, a.aggRefs, a.ectx)
+		row.reducers.consumeAsPartial(this, a.aggRefs, batch)
 	} else {
-		row.reducers.apply(a.zctx, a.ectx, a.aggs, this)
+		row.reducers.apply(a.zctx, batch, a.aggs, this)
 	}
 	return nil
 }
@@ -388,9 +377,8 @@ func (a *Aggregator) spillTable(eof bool, ref zbuf.Batch) error {
 	if err != nil || batch == nil {
 		return err
 	}
-	defer batch.Unref()
 	if a.spiller == nil {
-		a.spiller, err = spill.NewMergeSort(a.zctx, a.keysComparator)
+		a.spiller, err = spill.NewMergeSort(a.keysComparator)
 		if err != nil {
 			return err
 		}
@@ -401,11 +389,10 @@ func (a *Aggregator) spillTable(eof bool, ref zbuf.Batch) error {
 		return err
 	}
 	if !eof && a.inputDir != 0 {
-		arena := zed.NewArena()
-		defer arena.Unref()
-		ectx := expr.NewContextWithVars(arena, ref.Vars())
-		val := a.keyExprs[0].Eval(ectx, recs[len(recs)-1])
+		val := a.keyExprs[0].Eval(batch, recs[len(recs)-1])
 		if !val.IsError() {
+			// pass volatile zed.Value since updateMaxSpillKey will make
+			// a copy if needed.
 			a.updateMaxSpillKey(val)
 		}
 	}
@@ -415,17 +402,15 @@ func (a *Aggregator) spillTable(eof bool, ref zbuf.Batch) error {
 // updateMaxTableKey is called with a volatile zed.Value to update the
 // max value seen in the table for the streaming logic when the input is sorted.
 func (a *Aggregator) updateMaxTableKey(val zed.Value) zed.Value {
-	if a.maxTableKey.IsZero() || a.valueCompare(val, a.maxTableKey) > 0 {
-		a.maxTableKeyArena.Reset()
-		a.maxTableKey = val.Copy(a.maxTableKeyArena)
+	if a.maxTableKey == nil || a.valueCompare(val, *a.maxTableKey) > 0 {
+		a.maxTableKey = val.Copy().Ptr()
 	}
-	return a.maxTableKey
+	return *a.maxTableKey
 }
 
 func (a *Aggregator) updateMaxSpillKey(v zed.Value) {
-	if a.maxSpillKey.IsZero() || a.valueCompare(v, a.maxSpillKey) > 0 {
-		a.maxSpillKeyArena.Reset()
-		a.maxSpillKey = v.Copy(a.maxSpillKeyArena)
+	if a.maxSpillKey == nil || a.valueCompare(v, *a.maxSpillKey) > 0 {
+		a.maxSpillKey = v.Copy().Ptr()
 	}
 }
 
@@ -434,9 +419,6 @@ func (a *Aggregator) updateMaxSpillKey(v zed.Value) {
 // the input is sorted in the primary key, Results can be called
 // before eof, and keys that are completed will returned.
 func (a *Aggregator) nextResult(eof bool, batch zbuf.Batch) (zbuf.Batch, error) {
-	if batch == nil {
-		return nil, nil
-	}
 	if a.spiller == nil {
 		return a.readTable(eof, a.partialsOut, batch)
 	}
@@ -454,24 +436,21 @@ func (a *Aggregator) readSpills(eof bool, batch zbuf.Batch) (zbuf.Batch, error)
 	if !eof && a.inputDir == 0 {
 		return nil, nil
 	}
-	arena := zed.NewArena()
-	defer arena.Unref()
-	ectx := expr.NewContextWithVars(arena, batch.Vars())
 	for len(recs) < op.BatchLen {
 		if !eof && a.inputDir != 0 {
-			rec, err := a.spiller.Peek(arena)
+			rec, err := a.spiller.Peek()
 			if err != nil {
 				return nil, err
 			}
 			if rec == nil {
 				break
 			}
-			keyVal := a.keyExprs[0].Eval(ectx, *rec)
-			if !keyVal.IsError() && a.valueCompare(keyVal, a.maxSpillKey) >= 0 {
+			keyVal := a.keyExprs[0].Eval(batch, *rec)
+			if !keyVal.IsError() && a.valueCompare(keyVal, *a.maxSpillKey) >= 0 {
 				break
 			}
 		}
-		rec, err := a.nextResultFromSpills(ectx)
+		rec, err := a.nextResultFromSpills(batch)
 		if err != nil {
 			return nil, err
 		}
@@ -483,7 +462,7 @@ func (a *Aggregator) readSpills(eof bool, batch zbuf.Batch) (zbuf.Batch, error)
 	if len(recs) == 0 {
 		return nil, nil
 	}
-	return zbuf.NewBatch(arena, recs, batch, batch.Vars()), nil
+	return zbuf.NewBatch(batch, recs), nil
 }
 
 func (a *Aggregator) nextResultFromSpills(ectx expr.Context) (*zed.Value, error) {
@@ -497,7 +476,7 @@ func (a *Aggregator) nextResultFromSpills(ectx expr.Context) (*zed.Value, error)
 	row := newValRow(a.aggs)
 	var firstRec *zed.Value
 	for {
-		rec, err := a.spiller.Peek(ectx.Arena())
+		rec, err := a.spiller.Peek()
 		if err != nil {
 			return nil, err
 		}
@@ -505,7 +484,7 @@ func (a *Aggregator) nextResultFromSpills(ectx expr.Context) (*zed.Value, error)
 			break
 		}
 		if firstRec == nil {
-			firstRec = rec.Copy(ectx.Arena()).Ptr()
+			firstRec = rec.Copy().Ptr()
 		} else if a.keysComparator.Compare(*firstRec, *rec) != 0 {
 			break
 		}
@@ -528,9 +507,9 @@ func (a *Aggregator) nextResultFromSpills(ectx expr.Context) (*zed.Value, error)
 	for _, f := range row {
 		var v zed.Value
 		if a.partialsOut {
-			v = f.ResultAsPartial(a.zctx, ectx.Arena())
+			v = f.ResultAsPartial(a.zctx)
 		} else {
-			v = f.Result(a.zctx, ectx.Arena())
+			v = f.Result(a.zctx)
 		}
 		types = append(types, v.Type())
 		a.builder.Append(v.Bytes())
@@ -540,7 +519,7 @@ func (a *Aggregator) nextResultFromSpills(ectx expr.Context) (*zed.Value, error)
 	if err != nil {
 		return nil, err
 	}
-	return ectx.Arena().New(typ, bytes).Ptr(), nil
+	return zed.NewValue(typ, bytes).Ptr(), nil
 }
 
 // readTable returns a slice of records from the in-memory groupby
@@ -549,14 +528,12 @@ func (a *Aggregator) nextResultFromSpills(ectx expr.Context) (*zed.Value, error)
 // If partialsOut is true, it returns partial aggregation results as
 // defined by each agg.Function.ResultAsPartial() method.
 func (a *Aggregator) readTable(flush, partialsOut bool, batch zbuf.Batch) (zbuf.Batch, error) {
-	arena := zed.NewArena()
-	defer arena.Unref()
 	var recs []zed.Value
 	for key, row := range a.table {
 		if !flush && a.valueCompare == nil {
 			panic("internal bug: tried to fetch completed tuples on non-sorted input")
 		}
-		if !flush && a.valueCompare(row.groupval, a.maxTableKey) >= 0 {
+		if !flush && a.valueCompare(row.groupval, *a.maxTableKey) >= 0 {
 			continue
 		}
 		// To build the output record, we spin over the key values
@@ -579,9 +556,9 @@ func (a *Aggregator) readTable(flush, partialsOut bool, batch zbuf.Batch) (zbuf.
 		for _, f := range row.reducers {
 			var v zed.Value
 			if partialsOut {
-				v = f.ResultAsPartial(a.zctx, arena)
+				v = f.ResultAsPartial(a.zctx)
 			} else {
-				v = f.Result(a.zctx, arena)
+				v = f.Result(a.zctx)
 			}
 			types = append(types, v.Type())
 			a.builder.Append(v.Bytes())
@@ -591,7 +568,7 @@ func (a *Aggregator) readTable(flush, partialsOut bool, batch zbuf.Batch) (zbuf.
 		if err != nil {
 			return nil, err
 		}
-		recs = append(recs, arena.New(typ, zv))
+		recs = append(recs, zed.NewValue(typ, zv))
 		// Delete entries from the table as we create records, so
 		// the freed enries can be GC'd incrementally as we shift
 		// state from the table to the records.  Otherwise, when
@@ -603,7 +580,7 @@ func (a *Aggregator) readTable(flush, partialsOut bool, batch zbuf.Batch) (zbuf.
 	if len(recs) == 0 {
 		return nil, nil
 	}
-	return zbuf.NewBatch(arena, recs, batch, batch.Vars()), nil
+	return zbuf.NewBatch(batch, recs), nil
 }
 
 func (a *Aggregator) lookupRecordType(types []zed.Type) *zed.TypeRecord {
