@@ -102,9 +102,6 @@ func TestGroupbyStreamingSpill(t *testing.T) {
 		data = append(data, fmt.Sprintf("{ts:%s,ip:1.1.1.%d}", nano.Unix(int64(t), 0), i%uniqueIpsPerTs))
 	}
 
-	arena := zed.NewArena()
-	defer arena.Unref()
-
 	runOne := func(inputSortKey string) []string {
 		proc, _, err := compiler.Parse("count() by every(1s), ip")
 		assert.NoError(t, err)
@@ -124,7 +121,7 @@ func TestGroupbyStreamingSpill(t *testing.T) {
 				}
 			},
 		}
-		sortKey := order.NewSortKey(order.Asc, field.List{field.Path{inputSortKey}})
+		sortKey := order.NewSortKey(order.Asc, field.Path{inputSortKey})
 		query, err := newQueryOnOrderedReader(context.Background(), zctx, proc, cr, sortKey)
 		require.NoError(t, err)
 		defer query.Pull(true)
