@@ -31,9 +31,6 @@ type Op struct {
 	joinKey     *zed.Value
 	joinSet     []zed.Value
 	types       map[int]map[int]*zed.TypeRecord
-
-	joinKeyArena *zed.Arena
-	joinSetArena *zed.Arena
 }
 
 func New(rctx *runtime.Context, anti, inner bool, left, right zbuf.Puller, leftKey, rightKey expr.Evaluator,
@@ -48,34 +45,34 @@ func New(rctx *runtime.Context, anti, inner bool, left, right zbuf.Puller, leftK
 	var err error
 	// Add sorts if needed.
 	if !leftDir.HasOrder(o) {
-		left, err = sort.New(rctx, left, []expr.Evaluator{leftKey}, o, false, resetter)
+		s := expr.NewSortEvaluator(leftKey, o)
+		left, err = sort.New(rctx, left, []expr.SortEvaluator{s}, false, false, resetter)
 		if err != nil {
 			return nil, err
 		}
 	}
 	if !rightDir.HasOrder(o) {
-		right, err = sort.New(rctx, right, []expr.Evaluator{rightKey}, o, false, resetter)
+		s := expr.NewSortEvaluator(rightKey, o)
+		right, err = sort.New(rctx, right, []expr.SortEvaluator{s}, false, false, resetter)
 		if err != nil {
 			return nil, err
 		}
 	}
 	ctx, cancel := context.WithCancel(rctx.Context)
 	return &Op{
-		rctx:         rctx,
-		anti:         anti,
-		inner:        inner,
-		ctx:          ctx,
-		cancel:       cancel,
-		getLeftKey:   leftKey,
-		getRightKey:  rightKey,
-		left:         newPuller(left, ctx),
-		right:        zio.NewPeeker(newPuller(right, ctx)),
-		resetter:     resetter,
-		compare:      expr.NewValueCompareFn(o, true),
-		cutter:       expr.NewCutter(rctx.Zctx, lhs, rhs),
-		types:        make(map[int]map[int]*zed.TypeRecord),
-		joinKeyArena: zed.NewArena(),
-		joinSetArena: zed.NewArena(),
+		rctx:        rctx,
+		anti:        anti,
+		inner:       inner,
+		ctx:         ctx,
+		cancel:      cancel,
+		getLeftKey:  leftKey,
+		getRightKey: rightKey,
+		left:        newPuller(left, ctx),
+		right:       zio.NewPeeker(newPuller(right, ctx)),
+		resetter:    resetter,
+		compare:     expr.NewValueCompareFn(o, true),
+		cutter:      expr.NewCutter(rctx.Zctx, lhs, rhs),
+		types:       make(map[int]map[int]*zed.TypeRecord),
 	}, nil
 }
 
@@ -86,10 +83,9 @@ func (o *Op) Pull(done bool) (zbuf.Batch, error) {
 		go o.left.run()
 		go o.right.Reader.(*puller).run()
 	})
-	arena := zed.NewArena()
-	// See #3366
-	ectx := expr.NewContext(arena)
 	var out []zed.Value
+	// See #3366
+	ectx := expr.NewContext()
 	for {
 		leftRec, err := o.left.Read()
 		if err != nil {
@@ -101,7 +97,7 @@ func (o *Op) Pull(done bool) (zbuf.Batch, error) {
 				return nil, nil
 			}
 			//XXX See issue #3427.
-			return zbuf.NewArray(arena, out), nil
+			return zbuf.NewArray(out), nil
 		}
 		key := o.getLeftKey.Eval(ectx, *leftRec)
 		if key.IsMissing() {
@@ -110,7 +106,7 @@ func (o *Op) Pull(done bool) (zbuf.Batch, error) {
 			// left records that can eval the key expression.
 			continue
 		}
-		rightRecs, err := o.getJoinSet(ectx, key)
+		rightRecs, err := o.getJoinSet(key)
 		if err != nil {
 			return nil, err
 		}
@@ -118,7 +114,7 @@ func (o *Op) Pull(done bool) (zbuf.Batch, error) {
 			// Nothing to add to the left join.
 			// Accumulate this record for an outer join.
 			if !o.inner {
-				out = append(out, leftRec.Copy(arena))
+				out = append(out, leftRec.Copy())
 			}
 			continue
 		}
@@ -136,7 +132,7 @@ func (o *Op) Pull(done bool) (zbuf.Batch, error) {
 		// release the batch with and bypass GC.
 		for _, rightRec := range rightRecs {
 			cutRec := o.cutter.Eval(ectx, rightRec)
-			rec, err := o.splice(arena, *leftRec, cutRec)
+			rec, err := o.splice(*leftRec, cutRec)
 			if err != nil {
 				return nil, err
 			}
@@ -145,11 +141,12 @@ func (o *Op) Pull(done bool) (zbuf.Batch, error) {
 	}
 }
 
-func (o *Op) getJoinSet(ectx expr.Context, leftKey zed.Value) ([]zed.Value, error) {
+func (o *Op) getJoinSet(leftKey zed.Value) ([]zed.Value, error) {
 	if o.joinKey != nil && o.compare(leftKey, *o.joinKey) == 0 {
 		return o.joinSet, nil
 	}
 	// See #3366
+	ectx := expr.NewContext()
 	for {
 		rec, err := o.right.Peek()
 		if err != nil || rec == nil {
@@ -162,9 +159,13 @@ func (o *Op) getJoinSet(ectx expr.Context, leftKey zed.Value) ([]zed.Value, erro
 		}
 		cmp := o.compare(leftKey, rightKey)
 		if cmp == 0 {
-			o.joinKeyArena.Reset()
-			o.joinKey = leftKey.Copy(o.joinKeyArena).Ptr()
-			o.joinSet, err = o.readJoinSet(ectx, o.joinKey)
+			// Copy leftKey.Bytes since it might get reused.
+			if o.joinKey == nil {
+				o.joinKey = leftKey.Copy().Ptr()
+			} else {
+				o.joinKey.CopyFrom(leftKey)
+			}
+			o.joinSet, err = o.readJoinSet(o.joinKey)
 			return o.joinSet, err
 		}
 		if cmp < 0 {
@@ -183,10 +184,10 @@ func (o *Op) getJoinSet(ectx expr.Context, leftKey zed.Value) ([]zed.Value, erro
 // fillJoinSet is called when a join key has been found that matches
 // the current lefthand key.  It returns the all the subsequent records
 // from the righthand stream that match this key.
-func (o *Op) readJoinSet(ectx expr.Context, joinKey *zed.Value) ([]zed.Value, error) {
-	o.joinSetArena.Reset()
-	// See #3366
+func (o *Op) readJoinSet(joinKey *zed.Value) ([]zed.Value, error) {
 	var recs []zed.Value
+	// See #3366
+	ectx := expr.NewContext()
 	for {
 		rec, err := o.right.Peek()
 		if err != nil {
@@ -203,7 +204,7 @@ func (o *Op) readJoinSet(ectx expr.Context, joinKey *zed.Value) ([]zed.Value, er
 		if o.compare(key, *joinKey) != 0 {
 			return recs, nil
 		}
-		recs = append(recs, rec.Copy(o.joinSetArena))
+		recs = append(recs, rec.Copy())
 		o.right.Read()
 	}
 }
@@ -250,9 +251,9 @@ func (o *Op) combinedType(left, right *zed.TypeRecord) (*zed.TypeRecord, error)
 	return typ, nil
 }
 
-func (o *Op) splice(arena *zed.Arena, left, right zed.Value) (zed.Value, error) {
-	left = left.Under(arena)
-	right = right.Under(arena)
+func (o *Op) splice(left, right zed.Value) (zed.Value, error) {
+	left = left.Under()
+	right = right.Under()
 	typ, err := o.combinedType(zed.TypeRecordOf(left.Type()), zed.TypeRecordOf(right.Type()))
 	if err != nil {
 		return zed.Null, err
@@ -261,5 +262,5 @@ func (o *Op) splice(arena *zed.Arena, left, right zed.Value) (zed.Value, error)
 	bytes := make([]byte, n+len(right.Bytes()))
 	copy(bytes, left.Bytes())
 	copy(bytes[n:], right.Bytes())
-	return arena.New(typ, bytes), nil
+	return zed.NewValue(typ, bytes), nil
 }
