@@ -16,21 +16,20 @@ import (
 )
 
 func NewLakeMetaScanner(ctx context.Context, zctx *zed.Context, r *lake.Root, meta string) (zbuf.Scanner, error) {
-	arena := zed.NewArena()
 	var vals []zed.Value
 	var err error
 	switch meta {
 	case "pools":
-		vals, err = r.BatchifyPools(ctx, zctx, arena, nil)
+		vals, err = r.BatchifyPools(ctx, zctx, nil)
 	case "branches":
-		vals, err = r.BatchifyBranches(ctx, zctx, arena, nil)
+		vals, err = r.BatchifyBranches(ctx, zctx, nil)
 	default:
 		return nil, fmt.Errorf("unknown lake metadata type: %q", meta)
 	}
 	if err != nil {
 		return nil, err
 	}
-	return zbuf.NewScanner(ctx, zbuf.NewArray(arena, vals), nil)
+	return zbuf.NewScanner(ctx, zbuf.NewArray(vals), nil)
 }
 
 func NewPoolMetaScanner(ctx context.Context, zctx *zed.Context, r *lake.Root, poolID ksuid.KSUID, meta string) (zbuf.Scanner, error) {
@@ -38,20 +37,19 @@ func NewPoolMetaScanner(ctx context.Context, zctx *zed.Context, r *lake.Root, po
 	if err != nil {
 		return nil, err
 	}
-	arena := zed.NewArena()
 	var vals []zed.Value
 	switch meta {
 	case "branches":
 		m := zson.NewZNGMarshalerWithContext(zctx)
 		m.Decorate(zson.StylePackage)
-		vals, err = p.BatchifyBranches(ctx, zctx, arena, nil, m, nil)
+		vals, err = p.BatchifyBranches(ctx, zctx, nil, m, nil)
 		if err != nil {
 			return nil, err
 		}
 	default:
 		return nil, fmt.Errorf("unknown pool metadata type: %q", meta)
 	}
-	return zbuf.NewScanner(ctx, zbuf.NewArray(arena, vals), nil)
+	return zbuf.NewScanner(ctx, zbuf.NewArray(vals), nil)
 }
 
 func NewCommitMetaScanner(ctx context.Context, zctx *zed.Context, r *lake.Root, poolID, commit ksuid.KSUID, meta string, pruner expr.Evaluator) (zbuf.Puller, error) {
@@ -77,12 +75,11 @@ func NewCommitMetaScanner(ctx context.Context, zctx *zed.Context, r *lake.Root,
 		}
 		return zbuf.NewScanner(ctx, zbuf.PullerReader(slicer), nil)
 	case "log":
-		arena := zed.NewArena()
-		tips, err := p.BatchifyBranchTips(ctx, zctx, arena, nil)
+		tips, err := p.BatchifyBranchTips(ctx, zctx, nil)
 		if err != nil {
 			return nil, err
 		}
-		tipsScanner, err := zbuf.NewScanner(ctx, zbuf.NewArray(arena, tips), nil)
+		tipsScanner, err := zbuf.NewScanner(ctx, zbuf.NewArray(tips), nil)
 		if err != nil {
 			return nil, err
 		}
@@ -104,7 +101,7 @@ func NewCommitMetaScanner(ctx context.Context, zctx *zed.Context, r *lake.Root,
 			return nil, err
 		}
 		vectors := commits.Vectors(snap)
-		reader, err := objectReader(ctx, zctx, vectors, p.SortKey.Order)
+		reader, err := objectReader(ctx, zctx, vectors, p.SortKeys.Primary().Order)
 		if err != nil {
 			return nil, err
 		}
@@ -115,7 +112,6 @@ func NewCommitMetaScanner(ctx context.Context, zctx *zed.Context, r *lake.Root,
 }
 
 func objectReader(ctx context.Context, zctx *zed.Context, snap commits.View, order order.Which) (zio.Reader, error) {
-	arena := zed.NewArena()
 	objects := snap.Select(nil, order)
 	m := zson.NewZNGMarshalerWithContext(zctx)
 	m.Decorate(zson.StylePackage)
@@ -123,8 +119,7 @@ func objectReader(ctx context.Context, zctx *zed.Context, snap commits.View, ord
 		if len(objects) == 0 {
 			return nil, nil
 		}
-		arena.Reset()
-		val, err := m.Marshal(arena, objects[0])
+		val, err := m.Marshal(objects[0])
 		objects = objects[1:]
 		return &val, err
 	}), nil
