@@ -91,8 +91,6 @@ func coerceVals(zctx *zed.Context, a, b vector.Any) (vector.Any, vector.Any, vec
 	//return id, ok
 }
 
-//XXX need to handle const here
-
 func promoteWider(id int, val vector.Any) vector.Any {
 	typ, err := zed.LookupPrimitiveByID(id)
 	if err != nil {
@@ -103,15 +101,25 @@ func promoteWider(id int, val vector.Any) vector.Any {
 		return val.Promote(typ)
 	case *vector.Uint:
 		return val.Promote(typ)
+	case *vector.Const:
+		var zedVal zed.Value
+		if zed.IsSigned(id) {
+			zedVal = zed.NewInt(typ, val.Value().Int())
+		} else {
+			zedVal = zed.NewUint(typ, val.Value().Uint())
+		}
+		return vector.NewConst(zedVal, val.Len(), val.Nulls)
 	case *vector.Dict:
 		promoted := val.Any.(vector.Promotable).Promote(typ)
 		return vector.NewDict(promoted, val.Index, val.Counts, val.Nulls)
+	case *vector.View:
+		promoted := val.Any.(vector.Promotable).Promote(typ)
+		return vector.NewView(val.Index, promoted)
 	default:
 		panic(fmt.Sprintf("promoteWider %T", val))
 	}
 }
 
-// XXX need to handle const here
 // XXX need to handle overflow errors
 func promoteToSigned(val vector.Any) vector.Any {
 	//XXX need wide variant here if we're going to support this semantic
@@ -123,11 +131,20 @@ func promoteToSigned(val vector.Any) vector.Any {
 		return val
 	case *vector.Uint:
 		return uintToInt(val)
+	case *vector.Const:
+		v, ok := ToInt(val.Value())
+		if !ok {
+			panic("ToInt failed")
+		}
+		return vector.NewConst(zed.NewInt64(v), val.Len(), val.Nulls)
 	case *vector.Dict:
 		promoted := promoteToSigned(val.Any)
 		return vector.NewDict(promoted, val.Index, val.Counts, val.Nulls)
+	case *vector.View:
+		promoted := promoteToSigned(val.Any)
+		return vector.NewView(val.Index, promoted)
 	default:
-		panic(fmt.Sprintf("intToFloat invalid type: %T", val))
+		panic(fmt.Sprintf("promoteToSigned %T", val))
 	}
 }
 
@@ -218,6 +235,16 @@ func intToFloat(val vector.Any) vector.Any {
 			f[k] = float64(vals[k])
 		}
 		return vector.NewFloat(zed.TypeFloat64, f, val.Nulls)
+	case *vector.Const:
+		f, ok := ToFloat(val.Value())
+		if !ok {
+			panic("ToFloat failed")
+		}
+		return vector.NewConst(zed.NewFloat64(f), val.Len(), val.Nulls)
+	case *vector.Dict:
+		return vector.NewDict(intToFloat(val.Any), val.Index, val.Counts, val.Nulls)
+	case *vector.View:
+		return vector.NewView(val.Index, intToFloat(val.Any))
 	default:
 		panic(fmt.Sprintf("intToFloat invalid type: %T", val))
 	}
