@@ -20,15 +20,17 @@ func NewCompare(zctx *zed.Context, lhs, rhs Evaluator, op string) *Compare {
 }
 
 func (c *Compare) Eval(val vector.Any) vector.Any {
-	return c.eval(c.lhs.Eval(val), c.rhs.Eval(val))
+	return vector.Apply(true, c.eval, c.lhs.Eval(val), c.rhs.Eval(val))
 }
 
-func (c *Compare) eval(lhs, rhs vector.Any) vector.Any {
-	lhs = vector.Under(lhs)
-	rhs = vector.Under(rhs)
-	lhs, rhs, _ = coerceVals(c.zctx, lhs, rhs)
+func (c *Compare) eval(vecs ...vector.Any) vector.Any {
+	lhs := vector.Under(vecs[0])
+	rhs := vector.Under(vecs[1])
+	lhs, rhs, errVal := coerceVals(c.zctx, lhs, rhs)
+	if errVal != nil {
+		return errVal
+	}
 	//XXX need to handle overflow (see sam)
-	//XXX unions and variants and single-value-with-error variant
 	//XXX nulls... for primitives we just do the compare but we need
 	// to or the nulls together
 	kind := vector.KindOf(lhs)
