@@ -85,10 +85,12 @@ func (l *loader) loadVector(g *errgroup.Group, paths Path, s shadow) {
 		s.mu.Lock()
 		vec := s.vec
 		if vec == nil {
-			vec = vector.NewConst(s.arena, s.val, s.length(), s.nulls.flat)
+			vec = vector.NewConst(s.val, s.length(), s.nulls.flat)
 			s.vec = vec
 		}
 		s.mu.Unlock()
+	case *error_:
+		l.loadVector(g, paths, s.vals)
 	case *named:
 		l.loadVector(g, paths, s.vals)
 	default:
@@ -285,7 +287,7 @@ func (l *loader) loadVals(typ zed.Type, s *primitive, nulls *vector.Bool) (vecto
 		offs[length] = off
 		return vector.NewTypeValue(offs, bytes, nulls), nil
 	case *zed.TypeOfNull:
-		return vector.NewConst(nil, zed.Null, s.length(), nil), nil
+		return vector.NewConst(zed.Null, s.length(), nil), nil
 	}
 	return nil, fmt.Errorf("internal error: vcache.loadPrimitive got unknown type %#v", typ)
 }
@@ -412,7 +414,7 @@ func empty(typ zed.Type, length uint32, nulls *vector.Bool) vector.Any {
 	case *zed.TypeOfType:
 		return vector.NewTypeValue(make([]uint32, length+1), nil, nulls)
 	case *zed.TypeOfNull:
-		return vector.NewConst(nil, zed.Null, length, nil)
+		return vector.NewConst(zed.Null, length, nil)
 	default:
 		panic(fmt.Sprintf("vcache.empty: unknown type encountered: %T", typ))
 	}
@@ -519,6 +521,9 @@ func (l *loader) fetchNulls(g *errgroup.Group, paths Path, s shadow) {
 		s.nulls.fetch(g, l.r)
 	case *const_:
 		s.nulls.fetch(g, l.r)
+	case *error_:
+		s.nulls.fetch(g, l.r)
+		l.fetchNulls(g, paths, s.vals)
 	case *named:
 		l.fetchNulls(g, paths, s.vals)
 	default:
@@ -573,6 +578,9 @@ func flattenNulls(paths Path, s shadow, parent *vector.Bool) {
 		s.nulls.flatten(parent)
 	case *const_:
 		s.nulls.flatten(parent)
+	case *error_:
+		s.nulls.flatten(parent)
+		flattenNulls(paths, s.vals, nil)
 	case *named:
 		flattenNulls(paths, s.vals, parent)
 	default:
