@@ -93,13 +93,21 @@ type primitive struct {
 type const_ struct {
 	mu sync.Mutex
 	count
-	arena *zed.Arena
-	val   zed.Value //XXX map this value? XXX, maybe wrap a shadow vector?, which could
+	val zed.Value //XXX map this value? XXX, maybe wrap a shadow vector?, which could
 	// have a named in it
 	vec   *vector.Const
 	nulls nulls
 }
 
+type error_ struct {
+	vals  shadow
+	nulls nulls
+}
+
+func (e *error_) length() uint32 {
+	return e.vals.length()
+}
+
 type named struct {
 	name string
 	vals shadow
@@ -141,6 +149,8 @@ func newShadow(m vng.Metadata, n *vng.Nulls, nullsCnt uint32) shadow {
 		}
 		nullsCnt += m.Count
 		return newShadow(m.Values, m, nullsCnt)
+	case *vng.Error:
+		return &error_{newShadow(m.Values, n, nullsCnt), nulls{meta: n}}
 	case *vng.Named:
 		return &named{m.Name, newShadow(m.Values, n, nullsCnt)}
 	case *vng.Record:
@@ -195,7 +205,6 @@ func newShadow(m vng.Metadata, n *vng.Nulls, nullsCnt uint32) shadow {
 	case *vng.Const:
 		return &const_{
 			count: count{m.Len(), nullsCnt},
-			arena: m.Arena,
 			val:   m.Value,
 			nulls: nulls{meta: n},
 		}
