@@ -2,6 +2,7 @@ package service
 
 import (
 	"errors"
+	"fmt"
 	"io"
 	"net/http"
 	"os"
@@ -20,12 +21,14 @@ import (
 	"github.com/brimdata/zed/lake/commits"
 	"github.com/brimdata/zed/lake/journal"
 	"github.com/brimdata/zed/lakeparse"
+	"github.com/brimdata/zed/order"
 	"github.com/brimdata/zed/pkg/storage"
 	"github.com/brimdata/zed/runtime"
 	"github.com/brimdata/zed/runtime/exec"
 	"github.com/brimdata/zed/runtime/sam/op"
 	"github.com/brimdata/zed/service/auth"
 	"github.com/brimdata/zed/service/srverr"
+	"github.com/brimdata/zed/zbuf"
 	"github.com/brimdata/zed/zio"
 	"github.com/brimdata/zed/zio/anyio"
 	"github.com/brimdata/zed/zio/csvio"
@@ -124,8 +127,8 @@ func handleQuery(c *Core, w *ResponseWriter, r *Request) {
 				}
 			}
 			if len(batch.Values()) == 0 {
-				if eoc, ok := batch.(*op.EndOfChannel); ok {
-					if err := writer.WhiteChannelEnd(int(*eoc)); err != nil {
+				if eoc, ok := batch.(*zbuf.EndOfChannel); ok {
+					if err := writer.WhiteChannelEnd(string(*eoc)); err != nil {
 						w.Logger.Warn("Error writing channel end", zap.Error(err))
 						handleError(err)
 						return
@@ -133,9 +136,9 @@ func handleQuery(c *Core, w *ResponseWriter, r *Request) {
 				}
 				continue
 			}
-			var cid int
-			batch, cid = op.Unwrap(batch)
-			if err := writer.WriteBatch(cid, batch); err != nil {
+			var label string
+			batch, label = zbuf.Unlabel(batch)
+			if err := writer.WriteBatch(label, batch); err != nil {
 				w.Logger.Warn("Error writing batch", zap.Error(err))
 				handleError(err)
 				return
@@ -241,9 +244,14 @@ func handlePoolStats(c *Core, w *ResponseWriter, r *Request) {
 func handlePoolPost(c *Core, w *ResponseWriter, r *Request) {
 	var req api.PoolPostRequest
 	if !r.Unmarshal(w, &req) {
+		fmt.Println("unmarshal.err")
 		return
 	}
-	pool, err := c.root.CreatePool(r.Context(), req.Name, req.SortKey, req.SeekStride, req.Thresh)
+	var sortKeys order.SortKeys
+	if len(req.SortKeys.Keys) > 0 {
+		sortKeys = append(sortKeys, order.NewSortKey(req.SortKeys.Order, req.SortKeys.Keys[0]))
+	}
+	pool, err := c.root.CreatePool(r.Context(), req.Name, sortKeys, req.SeekStride, req.Thresh)
 	if err != nil {
 		w.Error(err)
 		return
