@@ -6,7 +6,7 @@ import (
 )
 
 type Error struct {
-	Typ   zed.Type
+	Typ   *zed.TypeError
 	Vals  Any
 	Nulls *Bool
 }
@@ -15,7 +15,7 @@ var _ Any = (*Error)(nil)
 
 // XXX we shouldn't create empty fields... this was the old design, now
 // we create the entire vector structure and page in leaves, offsets, etc on demand
-func NewError(typ zed.Type, vals Any, nulls *Bool) *Error {
+func NewError(typ *zed.TypeError, vals Any, nulls *Bool) *Error {
 	return &Error{Typ: typ, Vals: vals, Nulls: nulls}
 }
 
@@ -37,14 +37,22 @@ func (e *Error) Serialize(b *zcode.Builder, slot uint32) {
 }
 
 func NewStringError(zctx *zed.Context, msg string, len uint32) *Error {
-	arena := zed.NewArena()
-	vals := NewConst(arena, arena.NewString(msg), len, nil)
+	vals := NewConst(zed.NewString(msg), len, nil)
 	return &Error{Typ: zctx.LookupTypeError(zed.TypeString), Vals: vals}
 }
 
 func NewMissing(zctx *zed.Context, len uint32) *Error {
-	arena := zed.NewArena()
-	missing := zctx.Missing(arena)
-	vals := NewConst(arena, missing, len, nil)
-	return &Error{Typ: missing.Type(), Vals: vals}
+	missing := zctx.Missing()
+	vals := NewConst(missing, len, nil)
+	return &Error{Typ: missing.Type().(*zed.TypeError), Vals: vals}
+}
+
+func NewWrappedError(zctx *zed.Context, msg string, val Any) *Error {
+	recType := zctx.MustLookupTypeRecord([]zed.Field{
+		{Name: "message", Type: zed.TypeString},
+		{Name: "on", Type: val.Type()},
+	})
+	sval := NewConst(zed.NewString(msg), val.Len(), nil)
+	rval := NewRecord(recType, []Any{sval, val}, val.Len(), nil)
+	return &Error{Typ: zctx.LookupTypeError(recType), Vals: rval}
 }
