@@ -22,7 +22,8 @@ const (
 const (
 	FormFlat  = 0
 	FormDict  = 1
-	FormConst = 2
+	FormView  = 2
+	FormConst = 3
 )
 
 //XXX might not need Kind...
@@ -41,6 +42,8 @@ func KindOf(v Any) Kind {
 		return KindString
 	case *Dict:
 		return KindOf(v.Any)
+	case *View:
+		return KindOf(v.Any)
 	case *Const:
 		return KindOfType(v.Value().Type())
 	default:
@@ -67,9 +70,9 @@ func KindFromString(v string) Kind {
 
 func KindOfType(typ zed.Type) Kind {
 	switch zed.TypeUnder(typ).(type) {
-	case *zed.TypeOfInt16, *zed.TypeOfInt32, *zed.TypeOfInt64, *zed.TypeOfDuration:
+	case *zed.TypeOfInt16, *zed.TypeOfInt32, *zed.TypeOfInt64, *zed.TypeOfDuration, *zed.TypeOfTime:
 		return KindInt
-	case *zed.TypeOfUint16, *zed.TypeOfUint32, *zed.TypeOfUint64, *zed.TypeOfTime:
+	case *zed.TypeOfUint16, *zed.TypeOfUint32, *zed.TypeOfUint64:
 		return KindUint
 	case *zed.TypeOfFloat16, *zed.TypeOfFloat32, *zed.TypeOfFloat64:
 		return KindFloat
@@ -89,6 +92,8 @@ func FormOf(v Any) (Form, bool) {
 		return FormFlat, true
 	case *Dict:
 		return FormDict, true
+	case *View:
+		return FormView, true
 	case *Const:
 		return FormConst, true
 	default:
@@ -102,6 +107,8 @@ func (f Form) String() string {
 		return "Flat"
 	case FormDict:
 		return "Dict"
+	case FormView:
+		return "View"
 	case FormConst:
 		return "Const"
 	default:
