@@ -45,3 +45,27 @@ func (s *String) Serialize(b *zcode.Builder, slot uint32) {
 		b.Append(zed.EncodeString(s.Value(slot)))
 	}
 }
+
+func StringValue(val Any, slot uint32) (string, bool) {
+	switch val := val.(type) {
+	case *String:
+		if val.Nulls.Value(slot) {
+			return "", true
+		}
+		return val.Value(slot), false
+	case *Const:
+		if val.Nulls.Value(slot) {
+			return "", true
+		}
+		s, _ := val.AsString()
+		return s, false
+	case *Dict:
+		if val.Nulls.Value(slot) {
+			return "", true
+		}
+		return StringValue(val.Any, uint32(val.Index[slot]))
+	case *View:
+		return StringValue(val.Any, val.Index[slot])
+	}
+	panic(val)
+}
