@@ -18,6 +18,14 @@ func NewUint(typ zed.Type, values []uint64, nulls *Bool) *Uint {
 	return &Uint{Typ: typ, Values: values, Nulls: nulls}
 }
 
+func NewUintEmpty(typ zed.Type, length uint32, nulls *Bool) *Uint {
+	return NewUint(typ, make([]uint64, 0, length), nulls)
+}
+
+func (u *Uint) Append(v uint64) {
+	u.Values = append(u.Values, v)
+}
+
 func (u *Uint) Type() zed.Type {
 	return u.Typ
 }
@@ -26,6 +34,10 @@ func (u *Uint) Len() uint32 {
 	return uint32(len(u.Values))
 }
 
+func (u *Uint) Value(slot uint32) uint64 {
+	return u.Values[slot]
+}
+
 func (u *Uint) Serialize(b *zcode.Builder, slot uint32) {
 	if u.Nulls != nil && u.Nulls.Value(slot) {
 		b.Append(nil)
