@@ -6,7 +6,6 @@ import (
 
 	"github.com/brimdata/zed/compiler/ast"
 	astzed "github.com/brimdata/zed/compiler/ast/zed"
-	"github.com/brimdata/zed/order"
 	"github.com/brimdata/zed/runtime/sam/expr/agg"
 	"github.com/brimdata/zed/runtime/sam/expr/function"
 	"github.com/brimdata/zed/zson"
@@ -486,15 +485,21 @@ func (c *canon) op(p ast.Op) {
 	case *ast.Sort:
 		c.next()
 		c.write("sort")
-		if p.Order == order.Desc {
+		if p.Reverse {
 			c.write(" -r")
 		}
 		if p.NullsFirst {
 			c.write(" -nulls first")
 		}
-		if len(p.Args) > 0 {
+		for k, s := range p.Args {
+			if k > 0 {
+				c.write(",")
+			}
 			c.space()
-			c.exprs(p.Args)
+			c.expr(s.Expr, "")
+			if s.Order != nil {
+				c.write(" %s", s.Order.Name)
+			}
 		}
 	case *ast.Load:
 		c.next()
@@ -627,6 +632,16 @@ func (c *canon) op(p ast.Op) {
 		c.next()
 		c.write("yield ")
 		c.exprs(p.Exprs)
+	case *ast.Output:
+		c.next()
+		c.write("output %s", p.Name.Name)
+	case *ast.Debug:
+		c.next()
+		c.write("debug")
+		if p.Expr != nil {
+			c.write(" ")
+			c.expr(p.Expr, "")
+		}
 	default:
 		c.open("unknown proc: %T", p)
 		c.close()
