@@ -3,7 +3,6 @@ package zfmt
 import (
 	"github.com/brimdata/zed/compiler/ast/dag"
 	astzed "github.com/brimdata/zed/compiler/ast/zed"
-	"github.com/brimdata/zed/order"
 	"github.com/brimdata/zed/zson"
 )
 
@@ -302,6 +301,22 @@ func (c *canonDAG) op(p dag.Op) {
 		c.ret()
 		c.flush()
 		c.write(")")
+	case *dag.Mirror:
+		c.next()
+		c.open("mirror (")
+		c.ret()
+		for _, seq := range []dag.Seq{p.Mirror, p.Main} {
+			c.ret()
+			c.write("=>")
+			c.open()
+			c.head = true
+			c.seq(seq)
+			c.close()
+		}
+		c.close()
+		c.ret()
+		c.flush()
+		c.write(")")
 	case *dag.Switch:
 		c.next()
 		c.open("switch ")
@@ -371,15 +386,19 @@ func (c *canonDAG) op(p dag.Op) {
 	case *dag.Sort:
 		c.next()
 		c.write("sort")
-		if p.Order == order.Desc {
+		if p.Reverse {
 			c.write(" -r")
 		}
 		if p.NullsFirst {
 			c.write(" -nulls first")
 		}
-		if len(p.Args) > 0 {
+		for i, s := range p.Args {
+			if i > 0 {
+				c.write(",")
+			}
 			c.space()
-			c.exprs(p.Args)
+			c.expr(s.Key, "")
+			c.write(" %s", s.Order)
 		}
 	case *dag.Load:
 		c.next()
@@ -492,8 +511,14 @@ func (c *canonDAG) op(p dag.Op) {
 		if p.Format != "" {
 			c.write(" format %s", p.Format)
 		}
-		if !p.SortKey.IsNil() {
-			c.write(" order %s", p.SortKey)
+		if !p.SortKeys.IsNil() {
+			c.write(" order")
+			for i, s := range p.SortKeys {
+				if i > 0 {
+					c.write(",")
+				}
+				c.write(" %s  %s", s.Key, s.Order)
+			}
 		}
 		if p.Filter != nil {
 			c.write(" filter (")
@@ -527,6 +552,9 @@ func (c *canonDAG) op(p dag.Op) {
 		c.head = true
 		c.seq(p.Body)
 		c.close()
+	case *dag.Output:
+		c.next()
+		c.write("output %s", p.Name)
 	default:
 		c.next()
 		c.open("unknown proc: %T", p)
