@@ -11,6 +11,7 @@ import (
 	"time"
 
 	"github.com/brimdata/zed"
+	"github.com/brimdata/zed/compiler/optimizer/demand"
 	"github.com/brimdata/zed/pkg/storage"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
@@ -22,7 +23,7 @@ func TestOpenFifoCancelation(t *testing.T) {
 		errCh := make(chan error)
 		ctx, cancel := context.WithCancel(context.Background())
 		go func() {
-			_, err := Open(ctx, zed.NewContext(), storage.NewFileSystem(), path, ReaderOpts{})
+			_, err := Open(ctx, zed.NewContext(), storage.NewFileSystem(), path, demand.All(), ReaderOpts{})
 			errCh <- err
 		}()
 		time.Sleep(10 * time.Millisecond)
