@@ -1,7 +1,6 @@
 package csvio
 
 import (
-	"runtime"
 	"strings"
 	"testing"
 
@@ -11,9 +10,7 @@ import (
 
 func TestNewReaderUsesContextParameter(t *testing.T) {
 	zctx := zed.NewContext()
-	r := NewReader(zctx, strings.NewReader("f\n1\n"), ReaderOpts{})
-	defer runtime.KeepAlive(r)
-	rec, err := r.Read()
+	rec, err := NewReader(zctx, strings.NewReader("f\n1\n"), ReaderOpts{}).Read()
 	require.NoError(t, err)
 	typ, err := zctx.LookupType(rec.Type().ID())
 	require.NoError(t, err)
