@@ -7,6 +7,6 @@ import (
 )
 
 // NewWriter returns a writer to w.
-func NewWriter(w io.WriteCloser) (*vng.Writer, error) {
+func NewWriter(w io.WriteCloser) *vng.Writer {
 	return vng.NewWriter(w)
 }
