@@ -16,19 +16,12 @@ type Writer struct {
 	header
 	flattener *expr.Flattener
 	typ       *zed.TypeRecord
-	zctx      *zed.Context
-	arena     *zed.Arena
-	mapper    *zed.Mapper
 }
 
 func NewWriter(w io.WriteCloser) *Writer {
-	zctx := zed.NewContext()
 	return &Writer{
 		writer:    w,
-		flattener: expr.NewFlattener(zctx),
-		zctx:      zctx,
-		arena:     zed.NewArena(),
-		mapper:    zed.NewMapper(zctx),
+		flattener: expr.NewFlattener(zed.NewContext()),
 	}
 }
 
@@ -37,12 +30,11 @@ func (w *Writer) Close() error {
 }
 
 func (w *Writer) Write(r zed.Value) error {
-	w.arena.Reset()
-	r, err := w.flattener.Flatten(w.arena, r)
+	r, err := w.flattener.Flatten(r)
 	if err != nil {
 		return err
 	}
-	path := r.Deref(w.arena, "_path").AsString()
+	path := r.Deref("_path").AsString()
 	if r.Type() != w.typ || path != w.Path {
 		if err := w.writeHeader(r, path); err != nil {
 			return err
@@ -61,11 +53,7 @@ func (w *Writer) Write(r zed.Value) error {
 			w.buf.WriteByte('\t')
 		}
 		needSeparator = true
-		typ, err := w.mapper.Enter(f.Type)
-		if err != nil {
-			return err
-		}
-		w.buf.WriteString(FormatValue(w.arena, w.arena.New(typ, bytes)))
+		w.buf.WriteString(FormatValue(zed.NewValue(f.Type, bytes)))
 	}
 	w.buf.WriteByte('\n')
 	_, err = w.writer.Write(w.buf.Bytes())
